@@ -28,6 +28,7 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using System.Text;
 using System.Threading;
@@ -91,9 +92,12 @@ namespace Beagle {
 
 		// Is this being indexed because of crawling or other
 		// background activity?
-		// If crawl is set, then the underlying file will be flushed
+		// If set, then the underlying file will be flushed
 		// from buffer cache as soon as it is indexed.
-		private bool crawled = true;
+		// Set it to true when the buffer cache should not
+		// be disturbed due to this indexable (e.g. during
+		// crawling).
+		private bool flush_buffer_cache = false;
 
 		// Is this object inherently contentless?
 		private bool no_content = false;
@@ -277,10 +281,10 @@ namespace Beagle {
 			get { return ! DeleteContent && ContentUri.IsFile && ParentUri == null; }
 		}
 
-		[XmlAttribute]
-		public bool Crawled {
-			get { return crawled; }
-			set { crawled = value; }
+		[XmlAttribute ("Crawled")]
+		public bool FlushBufferCache {
+			get { return flush_buffer_cache; }
+			set { flush_buffer_cache = value; }
 		}
 
 		[XmlAttribute]
@@ -417,6 +421,17 @@ namespace Beagle {
 			binary_stream = stream;
 		}
 
+#if ENABLE_RDF_ADAPTER
+		// List of links found in the content while indexing
+		private IList<string> links = null;
+
+		[XmlIgnore]
+		public IList<string> Links {
+			get { return links; }
+			set { links = value; }
+		}
+#endif
+
 		[XmlArrayItem (ElementName="Property", Type=typeof (Property))]
 		public ArrayList Properties {
 			get { return properties; }
