@@ -40,7 +40,6 @@ using Beagle.Util;
 
 namespace Beagle.Filters {
 
-	[PropertyKeywordMapping (Keyword="inarchive", PropertyName="fixme:inside_archive", IsKeyword=true, Description="Use 'inarchive:true' for files inside an archive.")]
 	public class FilterArchive : Beagle.Daemon.Filter {
 
 		internal delegate ArchiveEntry GetNextEntry ();
@@ -78,6 +77,7 @@ namespace Beagle.Filters {
 		}
 
 		bool setup_done = false;
+		string archive_type = "zip";
 
 		private void SetupArchiveStream ()
 		{
@@ -88,32 +88,38 @@ namespace Beagle.Filters {
 			case "application/zip":
 				archive_stream = new ZipInputStream (Stream);
 				get_next_entry = GetNextEntryZip;
+				archive_type = "zip";
 				break;
 
 			case "application/x-bzip-compressed-tar":
 				archive_stream = new TarInputStream (new BZip2InputStream (Stream));
 				get_next_entry = GetNextEntryTar;
+				archive_type = "tar-bz2";
 				break;
 
 			case "application/x-compressed-tar":
 			case "application/x-tgz":
 				archive_stream = new TarInputStream (new GZipInputStream (Stream));
 				get_next_entry = GetNextEntryTar;
+				archive_type = "tar-gz";
 				break;
 
 			case "application/x-tar":
 				archive_stream = new TarInputStream (Stream);
 				get_next_entry = GetNextEntryTar;
+				archive_type = "tar";
 				break;
 
 			case "application/x-gzip":
 				archive_stream = new GZipInputStream (Stream);
 				get_next_entry = GetNextEntrySingle;
+				archive_type = "gzip";
 				break;
 
 			case "application/x-bzip":
 				archive_stream = new BZip2InputStream (Stream);
 				get_next_entry = GetNextEntrySingle;
+				archive_type = "bz2";
 				break;
 
 			default:
@@ -165,15 +171,10 @@ namespace Beagle.Filters {
 			++count;
 			total_size += a_entry.Size;
 
-			// FIXME: For nested archives, create uid:foo#bar
-			// instead of uid:foo#xxx#bar (avoid duplicates ?)
-			// FIXME: Construct correct Uri
-			// 1. Use Uri (foo, true): foo is not escaped. This creates serialization error for uris containing non-ascii character
-			// 2: Using Uri (foo): foo is escaped. This causes foo.zip#b#c (file c in archive b in archive foo) to be escaped to foo.zip#b%23c. This should be properly escaped back. Note that, file b#c in archive foo.zip would also be escaped to foo.zip#b%23c. If there is any problem about incorrect filename containing '#' in an archive, then System.Uri should be abandoned and a custom Beagle.Uri should be created which can handle multiple fragments.
-			// If you read the Uri related FIXMEs and hacks in beagled/*Queryable,
-			// then you would see why creating Beagle.Uri class over System.Uri
-			// is not at all a bad idea :)
-			child = new Indexable (new Uri (Indexable.Uri.ToString () + "#" + a_entry.Name));
+			// Add "#<escaped-path-to-entry>" to the end of the Indexable Uri
+			// So, file b#c in archive foo.zip becomes file:///foo.zip#b%23c
+			// And file c in archive b in archive foo.zip becomes file:///foo.zip#b#c
+			child = new Indexable (UriFu.AddFragment (Indexable.Uri, a_entry.Name, false));
 
 			child.CacheContent = true;
 			child.MimeType = a_entry.MimeType;
@@ -182,7 +183,11 @@ namespace Beagle.Filters {
 			child.ContentUri = UriFu.PathToFileUri (a_entry.TempFile);
 			child.DeleteContent = true;
 
+			// FIXME Remove fixme:inside_archive during Property Hack Week
+			// Replace most flag properties by value properties
 			child.AddProperty (Property.NewBool ("fixme:inside_archive", true));
+			// Use this instead of fixme:inside_archive
+			child.AddProperty (Property.NewKeyword ("archive:type", archive_type));
 
 			child.AddProperty (Property.NewKeyword ("fixme:relativeuri", a_entry.Name));
 			child.AddProperty (Property.New ("fixme:comment", a_entry.Comment));
