@@ -43,6 +43,7 @@ namespace Beagle.Filters {
 		public FilterDOC () 
 		{
 			SnippetMode = true;
+			SetFileType ("document");
 		}
 
 		protected override void RegisterSupportedTypes ()
@@ -100,6 +101,7 @@ namespace Beagle.Filters {
 			pc.Arguments = new string [] { exe, FileInfo.FullName };
 			pc.RedirectStandardOutput = true;
 			pc.RedirectStandardError = true;
+			pc.UseLangC = true;
 
 			// Let beagle-doc-extractor run for 90 CPU seconds, max.
 			pc.CpuLimit = 90;
@@ -145,6 +147,8 @@ namespace Beagle.Filters {
 
 		override protected void DoClose ()
 		{
+			base.DoClose ();
+
 			if (! pull_started)
 				return;
 
@@ -154,7 +158,7 @@ namespace Beagle.Filters {
 			
 			string line;
 			while ((line = pout.ReadLine ()) != null)
-				Log.Warn ("doc extractor [{0}]: {1}", Uri, line);
+				Log.Warn ("doc extractor [{0}]: {1}", Indexable.Uri, line);
 
 			pout.Close ();
 			pc.Close ();
