@@ -71,6 +71,7 @@ namespace Beagle.Filters {
 
 			try {
 				FileStream fs = File.Open (path, FileMode.Open, FileAccess.Read, FileShare.Read);
+				//FIXME: Use XmlSerializerFactory instead of creating a new serializer everytime
 				XmlSerializer serializer = new XmlSerializer (typeof (ExternalFilterInfoContainer));
 				ExternalFilterInfoContainer container = (ExternalFilterInfoContainer) serializer.Deserialize (fs);
 				fs.Close ();
@@ -90,6 +91,14 @@ namespace Beagle.Filters {
 				Logger.Log.Error (ex, "Unable to parse {0}", path);
 			}
 
+			// This should be higher than the versions of the shipped filters
+			// That would allow anyone to use an external filter instead of a shipped one
+			// Current max version is 5. 10 seems reasonably high.
+			SetVersion (10);
+		}
+
+		protected override void RegisterSupportedTypes ()
+		{
 			if (this.filters == null)
 				return;
 
@@ -124,7 +133,7 @@ namespace Beagle.Filters {
 					if (efi.Extensions == null)
 						continue;
 
-					if (ArrayFu.IndexOfString (efi.Extensions, extension) != -1)
+					if (Array.IndexOf<string> (efi.Extensions, extension) != -1)
 						return efi;
 				}
 			}
@@ -134,7 +143,7 @@ namespace Beagle.Filters {
 					if (efi.MimeTypes == null)
 						continue;
 
-					if (ArrayFu.IndexOfString (efi.MimeTypes, mime_type) != -1)
+					if (Array.IndexOf<string> (efi.MimeTypes, mime_type) != -1)
 						return efi;
 				}
 			}
@@ -172,6 +181,10 @@ namespace Beagle.Filters {
 			SafeProcess pc = new SafeProcess ();
 			pc.Arguments = argv;
 			pc.RedirectStandardOutput = true;
+			pc.UseLangC = true;
+
+			// Let the external filter run for 2 minutes, max.
+			pc.CpuLimit = 120;
 
 			try {
 				pc.Start ();
