@@ -29,11 +29,8 @@ using Galago;
 
 namespace Beagle.Util {
 
-	public class GalagoTools {
+	public static class GalagoTools {
 
-		private GalagoTools () {} // This class is static
-		private static Galago.Service service;
-	
 		public enum Status {
 			Available = 0,
 			Offline = 1,
@@ -41,12 +38,28 @@ namespace Beagle.Util {
 			Idle = 3,
 			NoStatus = -1,
 		};
-		
+
+		private static bool init;
+
+		static GalagoTools ()
+		{
+			// Init only once. I see no reason to init and uninit during every function invocation.
+			try {
+				init = Galago.Global.Init ("beagle-galago-presence");
+			} catch (DllNotFoundException) {
+				// FIXME: Catch Debian Galago packaging bug.
+				// Catching runtime exceptions is not right. Do here but monitor the distro bug.
+				init = false;
+				Console.Error.WriteLine ("Galago init failed. No buddy information will be provided.");
+			}
+		}
+
 		public static Status GetPresence (string service_id, string username)
 		{
-			if (! Galago.Global.Init ("beagle-galago-presence"))
+			if (! init)
 				return Status.NoStatus;
-			service = Galago.Global.GetService (service_id, Galago.Origin.Remote, true);
+
+			Galago.Service service = Galago.Global.GetService (service_id, Galago.Origin.Remote, true);
 			if (service == null)
 				return Status.NoStatus;
 
@@ -80,16 +93,16 @@ namespace Beagle.Util {
 						break;
 				}
 			}
-			Galago.Global.Uninit();
 
 			return user_status;
 		}
+
 		public static string GetIdleTime (string service_id, string username)
 		{
-			if (! Galago.Global.Init ("beagle-galago-presence"))
+			if (! init)
 				return null;
 
-			service = Galago.Global.GetService (service_id, Galago.Origin.Remote, true);
+			Galago.Service service = Galago.Global.GetService (service_id, Galago.Origin.Remote, true);
 			if (service == null)
 				return null;
 
@@ -105,7 +118,6 @@ namespace Beagle.Util {
 			
 			string str =  StringFu.DurationToPrettyString  ( DateTime.Now, presence.IdleStartTime);
 			
-			Galago.Global.Uninit();
 			return str;
 		}
 	}
