@@ -39,7 +39,7 @@ namespace Beagle.Util {
 		public delegate void EmptyQueueDelegate ();
 		public event EmptyQueueDelegate EmptyQueueEvent;
 
-		public static bool Debug = false;
+		private static bool Debug = false;
 
 		public enum Priority {
 
@@ -74,6 +74,11 @@ namespace Beagle.Util {
 			public string Creator;
 			public string Description;
 
+			// Current status, to be implemented by instances
+			protected virtual string StatusName {
+				get { return null; }
+			}
+
 			public object Source = null; // this is just an opaque identifier
 
 			public ITaskCollector Collector = null;
@@ -186,6 +191,9 @@ namespace Beagle.Util {
 
 			public void Schedule (Scheduler scheduler)
 			{
+				if (this.cancelled)
+					return; // do not schedule a cancelled task
+
 				// Increment the task groups the first
 				// time a task is scheduled.
 				if (this.scheduler == null)
@@ -220,14 +228,33 @@ namespace Beagle.Util {
 			}
 
 			public void Cancel ()
+			{
+				Cancel (null);
+			}
+
+			public void Cancel (string reason)
 			{
 				if (! cancelled) {
+					AddToCancelledTaskList (reason);
 					DecrementAllTaskGroups ();
 					Cleanup (); // clean up after cancelled tasks
 				}
 				cancelled = true;
 			}
 
+			private void AddToCancelledTaskList (string reason)
+			{
+				string task_desc = String.Format ("Cancelled task: {5}\n"  +
+								  "        Tag: {0}\n" +
+								  "    Creator: {1}\n" +
+								  "Description: {2}\n" +
+								  "   Priority: {3} ({4})\n", 
+								  Tag, Creator, Description, Priority, SubPriority,
+								  (reason == null ? String.Empty : reason));
+				if (scheduler != null)
+					scheduler.ReportCancelledTask (task_desc);
+			}
+
 			///////////////////////////////
 
 			// The Task's count keeps track of how many
@@ -239,6 +266,18 @@ namespace Beagle.Util {
 				get { return count; }
 			}
 
+			// Keeps track of how many times the task was executed but
+			// there was some exception
+			private int misfires = 0;
+
+			// Allow at most this many exceptions per task
+			// This is a bit high since some tasks like the generators are
+			// long running tasks. This should be a high enough number
+			// to tolerate some exceptional cases but stop the task
+			// in case it went into an infinite loop with exceptions.
+			// Ideally all exceptions should be caught and handled downstream.
+			const int MAX_TASK_EXCEPTION = 200;
+
 			///////////////////////////////
 			
 			public void SpawnChild (Task child_task)
@@ -267,19 +306,31 @@ namespace Beagle.Util {
 					try {
 						DoTaskReal ();
 					} catch (Exception ex) {
+						misfires ++;
 						Logger.Log.Warn (ex,
 								 "Caught exception in DoTaskReal\n" +
 								 "        Tag: {0}\n" +
-								 "    Creator: {0}\n" +
-								 "Description: {0}\n" +
-								 "   Priority: {0} ({1})", 
+								 "    Creator: {1}\n" +
+								 "Description: {2}\n" +
+								 "   Priority: {3} ({4})", 
 								 Tag, Creator, Description, Priority, SubPriority);
+						if (misfires >= MAX_TASK_EXCEPTION) {
+							Log.Warn ("More than {5} exceptions in DoTaskReal. Disabling further execution of task:\n" +
+								 "        Tag: {0}\n" +
+								 "    Creator: {1}\n" +
+								 "Description: {2}\n" +
+								 "   Priority: {3} ({4})", 
+								 Tag, Creator, Description, Priority, SubPriority, MAX_TASK_EXCEPTION);
+							Cancel ("Exceptions in DoTaskReal");
+						}
 					}
 					sw.Stop ();
 					if (Debug)
 						Logger.Log.Debug ("Finished task {0} in {1}", Tag, sw);
 
-					if (Reschedule) {
+					if (cancelled) {
+						return;
+					} else if (Reschedule) {
 						++count;
 						if (Debug)
 							Log.Debug ("Rescheduling task {0}", Tag);
@@ -306,6 +357,9 @@ namespace Beagle.Util {
 					DoCleanup ();
 				} catch (Exception ex) {
 					Logger.Log.Warn (ex, "Caught exception cleaning up task '{0}'", Tag);
+				} finally {
+					Reschedule = false;
+					scheduler = null;
 				}
 			}
 
@@ -381,6 +435,9 @@ namespace Beagle.Util {
 
 				if (Description != null)
 					sb.Append (Description).Append ('\n');
+
+				if (StatusName != null)
+					sb.Append ("Status: ").Append (StatusName).Append ('\n');
 			}
 		}
 
@@ -549,7 +606,7 @@ namespace Beagle.Util {
 
 		private Hashtable tasks_by_tag = new Hashtable ();
 		private int total_executed_task_count = 0;
-		
+
 		public void Add (Task task)
 		{
 			if (task == null)
@@ -567,8 +624,8 @@ namespace Beagle.Util {
 				// is being slammed with them.
 				if (task.Priority == Priority.Immediate) {
 					// Shift our times down by one
-					Array.Copy (last_immediate_times, 1, last_immediate_times, 0, 4);
-					last_immediate_times [4] = DateTime.Now;
+					Array.Copy (last_immediate_times, 1, last_immediate_times, 0, immediate_throttle_count - 1);
+					last_immediate_times [immediate_throttle_count - 1] = DateTime.Now;
 				}
 				
 				old_task = tasks_by_tag [task.Tag] as Task;
@@ -662,16 +719,19 @@ namespace Beagle.Util {
 		//
 
 		// FIXME: random magic constants
-		const double idle_threshold              = 5.314159 * 60; // probably should be longer
-		const double idle_ramp_up_time           = 5.271828 * 60; // probably should be longer
-		const double default_delayed_rate_factor =  9.03;         // work about 1/10th of the time
-		const double default_idle_rate_factor    = 2.097;         // work about 1/3rd of the time
-		const double maximum_delay               = 20;            // never wait for more than 20s
-		const double min_throttled_delay         = 1.5;           // never wait less than this when throttled
-		const double min_overloaded_delay        = 2.2;           // never wait less than this when there are many tasks
-		const int    task_overload_threshold     = 60;            // number of tasks to process before delaying
+		const double idle_threshold               = 5.314159 * 60;  // probably should be longer
+		const double idle_ramp_up_time            = 5.271828 * 60;  // probably should be longer
+		const double default_delayed_rate_factor  = 9.03;           // work about 1/10th of the time
+		const double default_throttle_rate_factor = 3.042;          // work about 1/4th of the time
+		const double default_idle_rate_factor     = 2.097;          // work about 1/3rd of the time
+		const double maximum_delay                = 20;             // never wait for more than 20s
+		const double min_throttled_delay          = 1.5;            // never wait less than this when throttled
+		const double min_overloaded_delay         = 2.2;            // never wait less than this when there are many tasks
+		const int    task_overload_threshold      = 15;             // number of tasks to process before delaying
+		const int    immediate_throttle_count     = 5;              // number of immediate tasks to consider before throttling
+		const double immediate_throttle_delta     = 4.1;            // amount of time between first and last tracked task
 
-		DateTime[] last_immediate_times = new DateTime [5];
+		DateTime[] last_immediate_times = new DateTime [immediate_throttle_count];
 
 		// The return value and duration_of_previous_task are both measured in seconds.
 		private double ComputeDelay (Priority priority_of_next_task,
@@ -692,7 +752,7 @@ namespace Beagle.Util {
 			bool need_throttle = false;
 
 			// Never speed up if we are using the battery.
-			if (idle_time > idle_threshold && ! SystemInformation.UsingBattery) {
+			if (idle_time > idle_threshold && BatteryMonitor.UsingAC) {
 				is_idle = true;
 				double t = (idle_time - idle_threshold) / idle_ramp_up_time;				     
 				idle_scale = (1 - Math.Min (t, 1.0));
@@ -704,21 +764,21 @@ namespace Beagle.Util {
 				rate_factor = 0;
 
 				if (last_immediate_times [0] != DateTime.MinValue) {
-					TimeSpan last_add_delta = DateTime.Now.Subtract (last_immediate_times [4]);
+					TimeSpan last_add_delta = DateTime.Now.Subtract (last_immediate_times [immediate_throttle_count - 1]);
 
 					// If less than a second has gone by since the
 					// last immediate task was added, there is
 					// still a torrent of events coming in, and we
 					// may need to throttle.
 					if (last_add_delta.TotalSeconds <= 1) {
-						TimeSpan between_add_delta = last_immediate_times [4].Subtract (last_immediate_times [0]);
+						TimeSpan between_add_delta = last_immediate_times [immediate_throttle_count - 1].Subtract (last_immediate_times [0]);
 
-						// At least 5 immediate tasks have been
-						// added in the last second.  We
-						// definitely need to throttle.
-						if (between_add_delta.TotalSeconds <= 1) {
+						// At least immediate_throttle_count tasks have been
+						// added in the last immediate_throttle_delta seconds.
+						// We definitely need to throttle.
+						if (between_add_delta.TotalSeconds <= immediate_throttle_delta) {
 							need_throttle = true;
-							rate_factor = idle_scale * default_idle_rate_factor;
+							rate_factor = idle_scale * default_throttle_rate_factor;
 						}
 					}
 				}
@@ -826,6 +886,9 @@ namespace Beagle.Util {
 						continue;
 					}
 
+					if (Debug)
+						Log.Debug ("Running Scheduler inner loop.  Pending tasks: {0}", tasks_by_tag.Count);
+
 					// Walk across our list of tasks and find
 					// the next one to execute.
 					DateTime now = DateTime.Now;
@@ -895,6 +958,9 @@ namespace Beagle.Util {
 					delay = ComputeDelay (next_task.Priority, duration_of_previous_task, executed_task_count);
 					delay = Math.Min (delay, (next_trigger_time - now).TotalSeconds);
 
+					if (Debug)
+						Log.Debug ("Computed a delay of {0:.00}s for next task ({1}: {2})", delay, next_task.Tag, next_task.Priority);
+
 					// Adjust by the time that has actually elapsed since the
 					// last task.
 					delay -= (now - end_time_of_previous_task).TotalSeconds;
@@ -904,6 +970,9 @@ namespace Beagle.Util {
 					if (delay > 0.001) {
 						TimeSpan span = TimeSpanFromSeconds (delay);
 
+						if (Debug)
+							Log.Debug ("Waiting {0:.00}s until the next task at {1}", span.TotalSeconds, now + span);
+
 						status_str = String.Format ("Waiting for next task at {0}", now + span);
 						Monitor.Wait (big_lock, span);
 						executed_task_count = 0;
@@ -1021,6 +1090,18 @@ namespace Beagle.Util {
 		
 		//////////////////////////////////////////////////////////////////////////////
 
+		// A list of descriptions of cancelled tasks
+		// Kept for display purposes; it is useful to know
+		// which tasks were cancelled and why.
+		private ArrayList cancelled_tasks = new ArrayList ();
+
+		internal void ReportCancelledTask (string description)
+		{
+			cancelled_tasks.Add (description);
+		}
+
+		//////////////////////////////////////////////////////////////////////////////
+
 		private static StringBuilder cached_sb = new StringBuilder ();
 		
 		public SchedulerInformation GetCurrentStatus ()
@@ -1070,6 +1151,11 @@ namespace Beagle.Util {
 					current_status.BlockedTasks.Add (cached_sb.ToString ());
 				}
 
+				// Add the cancelled tasks to blocked_task list for the time being
+				// This is avoid ABI change in libbeagle
+				foreach (string description in cancelled_tasks)
+					current_status.BlockedTasks.Add (description);
+
 				current_status.TotalTaskCount = total_executed_task_count;
 				current_status.StatusString = status_str;
 
