@@ -6,11 +6,12 @@ using System.Text;
 using SemWeb;
 
 namespace SemWeb {
-	public class N3Writer : RdfWriter {
+	public class N3Writer : RdfWriter, CanForgetBNodes {
 		TextWriter writer;
-		NamespaceManager ns = new NamespaceManager();
+		NamespaceManager2 ns = new NamespaceManager2();
 		bool hasWritten = false;
 		bool closed = false;
+		bool closeStream = false;
 		
 		string lastSubject = null, lastPredicate = null;
 		
@@ -19,13 +20,16 @@ namespace SemWeb {
 		
 		Formats format = Formats.Turtle;
 		
+		private const string xsdInteger = NS.XMLSCHEMA + "integer";
+		private const string xsdDouble = NS.XMLSCHEMA + "double";
+		
 		public enum Formats {
 			NTriples,
 			Turtle,
 			Notation3
 		}
 		
-		public N3Writer(string file) : this(GetWriter(file)) { }
+		public N3Writer(string file) : this(GetWriter(file)) { closeStream = true; }
 
 		public N3Writer(TextWriter writer) {
 			this.writer = writer;
@@ -38,7 +42,7 @@ namespace SemWeb {
 		public override void Add(Statement statement) {
 			if (statement.AnyNull) throw new ArgumentNullException();
 			WriteStatement2(URI(statement.Subject), URI(statement.Predicate),
-				statement.Object is Literal ? ((Literal)statement.Object).ToString() : URI((Entity)statement.Object));
+				statement.Object is Literal ? Literal((Literal)statement.Object) : URI((Entity)statement.Object));
 		}
 
 		public override void Close() {
@@ -48,9 +52,20 @@ namespace SemWeb {
 				writer.WriteLine(".");
 			closed = true;
 			hasWritten = false;
-			writer.Flush();
+			if (closeStream)
+				writer.Close();
+			else
+				writer.Flush();
 		}
 
+		private string Literal(Literal literal) {
+			#if !SILVERLIGHT
+			if (format == Formats.NTriples || literal.DataType == null) return literal.ToString();
+			if (literal.DataType == xsdInteger) return literal.ParseValue().ToString();
+			if (literal.DataType == xsdDouble && format == Formats.Notation3) return literal.ParseValue().ToString();
+			#endif
+			return literal.ToString();
+		}
 		
 		private string URI(Entity entity) {
 			if (entity is Variable && ((Variable)entity).LocalName != null)
@@ -82,7 +97,7 @@ namespace SemWeb {
 				if (ok)
 					return ":" + uri.Substring(len);
 			}
-			if (Format == Formats.NTriples || ns == null) return "<" + Escape(uri) + ">";
+			if (Format == Formats.NTriples) return "<" + Escape(uri) + ">";
 			
 			string ret = ns.Normalize(uri);
 			if (ret[0] != '<') return ret;
@@ -151,14 +166,21 @@ namespace SemWeb {
 			closed = false;
 			
 			// Write the prefix directives at the beginning.
-			if (!hasWritten && ns != null && !(Format == Formats.NTriples)) {
-				foreach (string prefix in ns.GetPrefixes()) {
+			if (ns.addedPrefixes.Count > 0 && !(Format == Formats.NTriples)) {
+				if (hasWritten) {
+					writer.Write(".\n");
+					lastSubject = null;
+					lastPredicate = null;
+					hasWritten = false; // really means whether a statement is "open", missing a period
+				}
+				foreach (string prefix in ns.addedPrefixes) {
 					writer.Write("@prefix ");
 					writer.Write(prefix);
 					writer.Write(": <");
 					writer.Write(ns.GetNamespace(prefix));
 					writer.Write("> .\n");
 				}
+				ns.addedPrefixes.Clear();
 			}
 
 			// Repeated subject.
@@ -187,7 +209,7 @@ namespace SemWeb {
 			
 			// Start a new statement.
 			} else {
-				if (hasWritten)
+				if (hasWritten) // finish the last statement
 					writer.Write(".\n");
 					
 				WriteThing(subj);
@@ -205,5 +227,18 @@ namespace SemWeb {
 			writer.Write(text);
 			writer.Write(" ");
 		}
+	
+		private class NamespaceManager2 : NamespaceManager {
+			public ArrayList addedPrefixes = new ArrayList();
+			public override void AddNamespace(string uri, string prefix) {
+				base.AddNamespace(uri, prefix);
+				addedPrefixes.Add(prefix);
+			}
+		}
+		
+		void CanForgetBNodes.ForgetBNode(BNode bnode) {
+			anonNames.Remove(bnode);
+			anonNameMap.Remove(bnode);
+		}
 	}
 }
