@@ -64,6 +64,16 @@ namespace Beagle.Daemon
 
 		/////////////////////////////////////////////////////////
 
+		// Removable index related options
+		static bool arg_removable = false;
+		static string mnt_dir = null;
+		static string volume_label = null;
+
+		/////////////////////////////////////////////////////////
+
+		// 1. Different handling of --disable-directories
+		const int MINOR_VERSION = 1;
+
 		// Files and directories that are allowed to be in the target
 		// directory before we blow it away.  If we encounter any file
 		// or dir not in this list, we'll bail out.
@@ -99,7 +109,7 @@ namespace Beagle.Daemon
 		static Queue pending_directories = new Queue ();
 		static IndexerRequest pending_request;
 		
-		const int BATCH_SIZE = 30;
+		const int BATCH_SIZE = Lucene.Net.Index.IndexWriter.DEFAULT_MAX_BUFFERED_DOCS;
 		
 		/////////////////////////////////////////////////////////
 		
@@ -215,7 +225,14 @@ namespace Beagle.Daemon
 					++i;
 					break;
 
+				case "--removable":
+					arg_removable = true;
+					break;
+
 				default:
+					if (arg.StartsWith ("-") || arg.StartsWith ("--"))
+						PrintUsage ();
+
 					string path = Path.IsPathRooted (arg) ? arg : Path.GetFullPath (arg);
 					if (path != "/" && path.EndsWith ("/"))
 						path = path.TrimEnd ('/');
@@ -293,23 +310,79 @@ namespace Beagle.Daemon
 					Environment.Exit (1);
 				}
 
+				bool prev_removable = static_index_config.GetOption ("Removable", false);
+				if (arg_removable != prev_removable) {
+					Log.Error ("Index previously created for {0}-removable path",
+						   (prev_removable ? "" : "non"));
+					Environment.Exit (1);
+				} else {
+					volume_label = static_index_config.GetOption ("VolumeLabel", null);
+				}
+
 				// If arg_source is not given, and prev_source is present, use prev_source
 				// as the arg_source. This is useful for re-running build-index without
 				// giving --arg_source for already existing static index
 				arg_source = prev_source;
 			}
 
+			// Removable media related options
+			if (arg_removable) {
+				if (pending_files.Count > 0) {
+					Log.Error ("Indexing individual files is not allowed for removable media.");
+					Environment.Exit (1);
+				} else if (pending_directories.Count > 1) {
+					Log.Error ("Indexing multiple root directories is not allowed for removable media.");
+					Environment.Exit (1);
+				}
+
+				mnt_dir = ((DirectoryInfo) pending_directories.Peek ()).FullName;
+				if (mnt_dir.Length != 1)
+					mnt_dir = mnt_dir.TrimEnd ('/');
+
+				// compute volume label
+				// (1) directory name if block.is_volume is false
+				// (2) hal volume.label if set
+				// (3) hal volume.uuid if set
+				Hal.Manager manager = new Hal.Manager (new Hal.Context ());
+				Hal.Device mnt_device = null;
+
+				foreach (Hal.Device device in manager.FindDeviceStringMatch ("volume.mount_point", mnt_dir))
+					mnt_device = device;
+
+				string new_volume_label = null;
+				if (mnt_device != null) {
+					new_volume_label = mnt_device.GetPropertyString ("volume.label");
+
+
+					if (String.IsNullOrEmpty (new_volume_label))
+						new_volume_label = mnt_device.GetPropertyString ("volume.uuid");
+				}
+
+				if (new_volume_label == null)
+					new_volume_label = ((DirectoryInfo) pending_directories.Peek ()).Name;
+
+				// Sanity check
+				// Volume label is part of the URI, so cannot be changed once set
+				if (volume_label == null) {
+					volume_label = new_volume_label;
+				} else if (volume_label != new_volume_label) {
+					Log.Error ("Volume label (earlier '{0}') changed (to '{1}')! You need to create a new index.", volume_label, new_volume_label);
+					Environment.Exit (1);
+				}
+			}
+
 			if (arg_source == null) {
 				DirectoryInfo dir = new DirectoryInfo (StringFu.SanitizePath (arg_output));
 				arg_source = dir.Name;
 			}
 
-			if (SystemInformation.UsingBattery && arg_disable_on_battery) {
+			if (! BatteryMonitor.UsingAC && arg_disable_on_battery) {
 				Log.Always ("Indexer is disabled when on battery power (--disable-on-battery)");
 				Environment.Exit (0);
 			}
 
-			string global_files_config = Path.Combine (Conf.GlobalDir, Conf.Names.FilesQueryableConfig + ".xml");
+			string global_files_config = Path.Combine (PathFinder.ConfigDataDir, "config-files");
+			global_files_config = Path.Combine (global_files_config, Conf.Names.FilesQueryableConfig + ".xml");
 			if (! File.Exists (global_files_config)) {
 				Log.Error ("Global configuration file not found {0}", global_files_config);
 				Environment.Exit (0);
@@ -338,7 +411,7 @@ namespace Beagle.Daemon
 			SystemPriorities.SetSchedulerPolicyBatch ();
 			SystemPriorities.Renice (19);
 			
-			driver = new LuceneIndexingDriver (arg_output, false);
+			driver = new LuceneIndexingDriver (arg_output, MINOR_VERSION, false);
 			driver.TextCache = (arg_cache_text) ? new TextCache (arg_output) : null;
 			if (driver.TextCache != null)
 				driver.TextCache.WorldReadable = true;
@@ -347,7 +420,16 @@ namespace Beagle.Daemon
 			fa_store = new FileAttributesStore (backing_fa_store);
 			
 			// Set up signal handlers
+#if MONO_1_9
+			Shutdown.SetupSignalHandlers (delegate (int signal)
+							{
+								if (signal == (int) Mono.Unix.Native.Signum.SIGINT ||
+								    signal == (int) Mono.Unix.Native.Signum.SIGTERM)
+									Shutdown.BeginShutdown ();
+							});
+#else
 			SetupSignalHandlers ();
+#endif
 
 			Thread monitor_thread = null;
 
@@ -379,6 +461,15 @@ namespace Beagle.Daemon
 				// The name of the source
 				static_index_config.SetOption ("Source", arg_source);
 				static_index_config ["Source"].Description = "Source of the static index";
+
+				if (arg_removable) {
+					static_index_config.SetOption ("VolumeLabel", volume_label);
+					static_index_config ["VolumeLabel"].Description = "Volume label of the removable source";
+
+					static_index_config.SetOption ("Removable", true);
+					static_index_config ["Removable"].Description = "Removable source";
+				}
+
 				Conf.SaveTo (static_index_config, config_file_path);
 			}
 
@@ -424,8 +515,7 @@ namespace Beagle.Daemon
 			while (pending_directories.Count > 0) {
 				DirectoryInfo dir = (DirectoryInfo) pending_directories.Dequeue ();
 
-				if (! arg_disable_directories)
-					AddToRequest (DirectoryToIndexable (dir, modified_directories));
+				AddToRequest (DirectoryToIndexable (dir, modified_directories));
 
 				try {
 					if (arg_recursive)
@@ -465,7 +555,7 @@ namespace Beagle.Daemon
 				ICollection all_dirent = GetAllItemsInDirectory (subdir);
 				foreach (Dirent info in all_dirent) {
 					// check if the item exists
-					if (File.Exists (info.FullName) || 
+					if ((! info.IsDirectory && File.Exists (info.FullName)) || 
 					    (info.IsDirectory && Directory.Exists (info.FullName)))
 						continue;
 
@@ -474,7 +564,7 @@ namespace Beagle.Daemon
 						modified_directories.Enqueue (new DirectoryInfo (info.FullName));
 					
 					// remove
-					Uri uri = UriFu.PathToFileUri (info.FullName);
+					Uri uri = PathToUri (info.FullName);
 					indexable = new Indexable (IndexableType.Remove, uri);
 					AddToRequest (indexable);
 				}
@@ -532,7 +622,6 @@ namespace Beagle.Daemon
 					Logger.Log.Debug ("Flushing driver, {0} items in queue", pending_request.Count);
 
 				reschedule = FlushIndexer (driver);
-				Console.WriteLine (reschedule);
 
 				// Super Lame Hack: gtk-sharp up to 2.10 requires a main loop
 				// to dispose of any managed wrappers around GObjects.  Since
@@ -629,20 +718,30 @@ namespace Beagle.Daemon
 
 		static Indexable FileToIndexable (FileInfo file)
 		{
-			if (!file.Exists || fa_store.IsUpToDateAndFiltered (file.FullName))
+			if (!file.Exists)
+				return null;
+			
+			if (fa_store.IsUpToDateAndFiltered (PathInIndex (file.FullName),
+							    FileSystem.GetLastWriteTimeUtc (file.FullName)))
 				return null;
 
 			// Create the indexable and add the standard properties we
 			// use in the FileSystemQueryable.
-			Uri uri = UriFu.PathToFileUri (file.FullName);
+			Uri uri = PathToUri (file.FullName);
 			Indexable indexable = new Indexable (uri);
 			indexable.Timestamp = file.LastWriteTimeUtc;
-			indexable.Crawled = true;
+			indexable.FlushBufferCache = true;
+			indexable.AddProperty (Property.NewUnsearched ("fixme:filesize", file.Length));
 			FSQ.AddStandardPropertiesToIndexable (indexable, file.Name, Guid.Empty, false);
 
 			// Store directory name in the index
 			string dirname = file.DirectoryName;
-			indexable.AddProperty (Property.NewUnsearched (ParentDirUriPropKey, UriFu.PathToFileUri (dirname)));
+			indexable.AddProperty (Property.NewUnsearched (Property.ParentDirUriPropKey, PathToUri (dirname)));
+
+			if (arg_removable) {
+				indexable.AddProperty (Property.NewKeyword ("beagle:RemovableVolume", volume_label));
+				indexable.ContentUri = UriFu.PathToFileUri (file.FullName);
+			}
 
 			return indexable;
 		}
@@ -654,33 +753,82 @@ namespace Beagle.Daemon
 
 			// Check if the directory information is stored in attributes store
 			// And if the mtime of the directory is same as that in the attributes store
-			FileAttributes attr = fa_store.Read (dir.FullName);
+			FileAttributes attr = fa_store.Read (PathInIndex (dir.FullName));
 
-			// If the directory exists in the fa store, then it is already indexed
+			// If the directory exists in the fa store, then it is already indexed.
 			if (attr != null) {
-				if (arg_delete && ! FileAttributesStore.IsUpToDate (dir.FullName, attr))
-					modified_directories.Enqueue (dir);
-				return null;
+				// If we don't care about deleted content then we are fine.
+				// If the attributes are up-to-date, then we are fine too.
+				if (! arg_delete || FileAttributesStore.IsUpToDate (attr, FileSystem.GetLastWriteTimeUtc (dir.FullName)))
+					return null;
+
+				// But the last write time needs to be uptodate to support enable-deletion,
+				// so we actually index the directories, even if --disable-directories
+				// is set.
+				modified_directories.Enqueue (dir);
 			}
 
 			// Create the indexable and add the standard properties we
 			// use in the FileSystemQueryable.
-			Uri uri = UriFu.PathToFileUri (dir.FullName);
+			Uri uri = PathToUri (dir.FullName);
 			Indexable indexable = new Indexable (uri);
 			indexable.MimeType = "inode/directory";
 			indexable.NoContent = true;
 			indexable.Timestamp = dir.LastWriteTimeUtc;
-			FSQ.AddStandardPropertiesToIndexable (indexable, dir.Name, Guid.Empty, false);
+
+			// Store the directory information in the index anyway, but if --disable-directories
+			// was passed, then do not store the names and other standard properties
+			// used during searching
+			if (! arg_disable_directories)
+				FSQ.AddStandardPropertiesToIndexable (indexable, dir.Name, Guid.Empty, false);
 
 			// Add directory name property
 			string dirname = dir.Parent.FullName;
-			indexable.AddProperty (Property.NewUnsearched (ParentDirUriPropKey, UriFu.PathToFileUri (dirname)));
+			indexable.AddProperty (Property.NewUnsearched (Property.ParentDirUriPropKey, PathToUri (dirname)));
 
-			indexable.AddProperty (Property.NewBool (IsDirectoryPropKey, true));
+			indexable.AddProperty (Property.NewBool (Property.IsDirectoryPropKey, true));
+
+			if (arg_removable)
+				indexable.AddProperty (Property.NewKeyword ("beagle:removable", volume_label));
 
 			return indexable;
 		}
-		
+
+		////////////////////////////////////////////////////
+		// Methods to generate the relative path and
+		// uri for removable sources.
+
+		static string PathInIndex (string fullpath)
+		{
+			if (! arg_removable)
+				return fullpath;
+
+			if (fullpath == mnt_dir)
+				return "/";
+
+			if (fullpath.IndexOf (mnt_dir) != 0) {
+				Log.Warn ("Outside mounted directory: {0}", fullpath);
+				return fullpath;
+			}
+
+			return fullpath.Remove (0, mnt_dir.Length);
+		}
+
+		static Uri PathToUri (string fullpath)
+		{
+			if (! arg_removable)
+				return UriFu.PathToFileUri (fullpath);
+
+			fullpath = PathInIndex (fullpath);
+
+			return new Uri (String.Format ("removable{0}{1}{2}",
+							Uri.SchemeDelimiter,
+							volume_label,
+							StringFu.HexEscape (fullpath)), true);
+		}
+
+		///////////////////////////////////////////////////
+
 		class Dirent {
 			private bool is_directory;
 			private string path;
@@ -719,18 +867,15 @@ namespace Beagle.Daemon
 			}
 		}
 
-		private const string ParentDirUriPropKey = "beagle:ParentDirUri";
-		private const string IsDirectoryPropKey = "beagle:IsDirectory";
-
 		// Returns a list of all files and directories in dir
 		static ICollection GetAllItemsInDirectory (DirectoryInfo dir)
 		{
 			// form the query
-			string parent_uri_str = UriFu.PathToFileUri (dir.FullName).ToString ();
+			string parent_uri_str = PathToUri (dir.FullName).ToString ();
+
 			// Instead of taking the painfull way of using BeagleAnalyzer, lets just add the prefix manually
-			//parent_uri_str = "_:" + parent_uri_str;
 			// LuceneCommon thinks exposing secret property type encoding is bad, I think so too... except for now
-			string key = "prop:k:" + ParentDirUriPropKey;
+			string key = "prop:k:" + Property.ParentDirUriPropKey;
 			//Logger.Log.Debug ("Querying for {0}={1}", parent_uri_str, key);
 			LNS.Query query = new LNS.TermQuery (new Term (key, parent_uri_str));
 
@@ -781,12 +926,13 @@ namespace Beagle.Daemon
 
 			path = doc.Get ("Uri");
 
-			string prop_key = "prop:k:" + IsDirectoryPropKey;
+			string prop_key = "prop:k:" + Property.IsDirectoryPropKey;
 			foreach (Field f in doc.Fields ()) {
 				if (f.Name () != prop_key)
 					continue;
 
-				is_dir = (f.StringValue ().Substring (2) == "true");
+				// Format of fields: from LuceneCommon.cs:AddPropertyToDocument
+				is_dir = (f.StringValue ().Substring (3) == "true");
 				break;
 			}
 
@@ -871,16 +1017,22 @@ namespace Beagle.Daemon
 				"  --tag [tag]\t\t\tTag index data for identification.\n" + 
 				"  --recursive\t\t\tCrawl source path recursivly.\n" + 
 				"  --enable-deletion\t\tRemove deleted files and directories from index.\n" +
+				"                   \t\tIndex should be created and always updated with this option.\n" +
 				"  --enable-text-cache\t\tBuild text-cache of documents used for snippets.\n" +
 				"  --disable-directories\t\tDon't add directories to the index.\n" +
 				"  --disable-filtering\t\tDisable all filtering of files. Only index attributes.\n" + 
 				"  --allow-pattern [pattern]\tOnly allow files that match the pattern to be indexed.\n" + 
 				"  --deny-pattern [pattern]\tKeep any files that match the pattern from being indexed.\n" + 
 				"  --disable-restart\t\tDon't restart when memory usage gets above a certain threshold.\n" +
-				"  --disable-on-battery\t\tDisable indexer while on battery power.\n" +
-				"  --debug\t\t\tEcho verbose debugging information.\n\n";
+				"  --disable-on-battery\t\tDisable indexer while on battery power.\n";
+
+			usage +="\n" +
+				"Removable media options:\n" +
+				"  --removable\t\t\tCreate a removable media.\n" +
+				"\n" +
+				" * Use --source [name] to specify a name for the removable media.\n" +
+				" * You can specify only one <path>. Paths will be stored relative to <path>.\n";
 
-			
 			Console.WriteLine (usage);
 			Environment.Exit (0);
 		}
