@@ -59,7 +59,11 @@ namespace Beagle.Daemon {
 
 		public string Path {
 			get { return path; }
-			set { path = value != null ? System.IO.Path.GetFullPath (value) : null; }
+			set { 
+				path = value != null ? System.IO.Path.GetFullPath (value) : null;
+				if (path != null && path != "/" && path.EndsWith ("/"))
+					path = path.TrimEnd ('/');
+			}
 		}
 
 		public DateTime LastWriteTime {
@@ -74,7 +78,7 @@ namespace Beagle.Daemon {
 		}
 
 		public bool HasFilterInfo {
-			get { return filter_name != null && filter_version > 0; }
+			get { return filter_name != null && filter_version >= 0; }
 		}
 		
 		public string FilterName {
