@@ -40,6 +40,12 @@ namespace Beagle.Daemon {
 			store_sqlite = new FileAttributesStore_Sqlite (directory, index_fingerprint);
 		}
 
+		public void Dispose ()
+		{
+			store_ea.Dispose ();
+			store_sqlite.Dispose ();
+		}
+
 		// We always have to query the Sqlite store first, because of our
 		// EA nightmare scenario: a file whose permissions or ownership get
 		// changed after the EAs have been attached.  Thus attributes in
@@ -50,19 +56,14 @@ namespace Beagle.Daemon {
 			FileAttributes attr;
 			attr = store_sqlite.Read (path);
 
-			if (attr!= null) {
+			if (attr != null) {
 				// If we have write access to the path but it has attributes
 				// stored in the sqlite file attributes db, we should attach them
 				// to the file with EAs and delete the record from the db.
-				// Check if we have write access ?
+
 				if (! FileSystem.IsWritable (path))
 					return attr;
 
-				// What are the other cases when writing an xattr would fail ?
-				// FIXME: Should also check if extended attributes is supported ?
-				// Can that be done without incurring much cost, in which case
-				// we may try to write it anyway ?
-
 				bool success = store_ea.Write (attr);
 				if (success)
 					store_sqlite.Drop (path);
@@ -73,7 +74,13 @@ namespace Beagle.Daemon {
 
 		public bool Write (FileAttributes attr)
 		{
-			return store_ea.Write (attr) || store_sqlite.Write (attr);
+			if (store_ea.Write (attr)) {
+				// Drop any now-outdated information from the sqlite store.
+				store_sqlite.Drop (attr.Path);
+
+				return true;
+			} else
+				return store_sqlite.Write (attr);
 		}
 
 		public void Drop (string path)
