@@ -1,9 +1,10 @@
 /*
- * Copyright 2005 The Apache Software Foundation
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  * http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -29,9 +30,11 @@ namespace Lucene.Net
 		
 		/// <summary>Return Lucene's package, including version information. </summary>
 		// {{Aroush-1.9}}
-		//// public static Package Get()
-		//// {
-		//// 	return typeof(LucenePackage).getPackage();
-		//// }
+        /*
+		public static Package Get()
+		{
+			return typeof(LucenePackage).getPackage();
+		}
+        */
 	}
 }
\ No newline at end of file
