@@ -1,9 +1,10 @@
 /*
- * Copyright 2004 The Apache Software Foundation
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  * http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -13,8 +14,11 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 using System;
+
 using IndexReader = Lucene.Net.Index.IndexReader;
+
 namespace Lucene.Net.Search
 {
 	
@@ -27,7 +31,7 @@ namespace Lucene.Net.Search
 	/// </author>
 	/// <since>   lucene 1.4
 	/// </since>
-	/// <version>  $Id: FieldCache.cs,v 1.2 2005/10/06 19:29:57 dsd Exp $
+	/// <version>  $Id: FieldCache.java 472959 2006-11-09 16:21:50Z yonik $
 	/// </version>
 	/// <summary>Expert: Stores term text values and document ordering data. </summary>
 	public class StringIndex
@@ -61,64 +65,90 @@ namespace Lucene.Net.Search
 	}
 	public interface FieldCache
 	{
-        /// <summary>Indicator for StringIndex values in the cache. </summary>
-        // NOTE: the value assigned to this constant must not be
-        // the same as any of those in SortField!!
-        //readonly static int STRING_INDEX = - 1;
-        /// <summary>Expert: The cache used internally by sorting and range query classes. </summary>
-        //readonly static FieldCache DEFAULT = new FieldCacheImpl();
-		
+		void Close(IndexReader reader);
 		
 		/// <summary>Checks the internal cache for an appropriate entry, and if none is
-		/// found, reads the terms in <code>Field</code> as integers and returns an array
+		/// found, reads the terms in <code>field</code> as integers and returns an array
 		/// of size <code>reader.maxDoc()</code> of the value each document
-		/// has in the given Field.
+		/// has in the given field.
 		/// </summary>
-		/// <param name="reader"> Used to get Field values.
+		/// <param name="reader"> Used to get field values.
 		/// </param>
-		/// <param name="Field">  Which Field contains the integers.
+		/// <param name="field">  Which field contains the integers.
 		/// </param>
-		/// <returns> The values in the given Field for each document.
+		/// <returns> The values in the given field for each document.
 		/// </returns>
 		/// <throws>  IOException  If any error occurs. </throws>
 		int[] GetInts(IndexReader reader, System.String field);
 		
-		/// <summary>Checks the internal cache for an appropriate entry, and if
-		/// none is found, reads the terms in <code>Field</code> as floats and returns an array
-		/// of size <code>reader.maxDoc()</code> of the value each document
-		/// has in the given Field.
+		/// <summary>Checks the internal cache for an appropriate entry, and if none is found,
+		/// reads the terms in <code>field</code> as integers and returns an array of
+		/// size <code>reader.maxDoc()</code> of the value each document has in the
+		/// given field.
 		/// </summary>
-		/// <param name="reader"> Used to get Field values.
+		/// <param name="reader"> Used to get field values.
 		/// </param>
-		/// <param name="Field">  Which Field contains the floats.
+		/// <param name="field">  Which field contains the integers.
 		/// </param>
-		/// <returns> The values in the given Field for each document.
+		/// <param name="parser"> Computes integer for string values.
+		/// </param>
+		/// <returns> The values in the given field for each document.
+		/// </returns>
+		/// <throws>  IOException  If any error occurs. </throws>
+		int[] GetInts(IndexReader reader, System.String field, IntParser parser);
+		
+		/// <summary>Checks the internal cache for an appropriate entry, and if
+		/// none is found, reads the terms in <code>field</code> as floats and returns an array
+		/// of size <code>reader.maxDoc()</code> of the value each document
+		/// has in the given field.
+		/// </summary>
+		/// <param name="reader"> Used to get field values.
+		/// </param>
+		/// <param name="field">  Which field contains the floats.
+		/// </param>
+		/// <returns> The values in the given field for each document.
 		/// </returns>
 		/// <throws>  IOException  If any error occurs. </throws>
 		float[] GetFloats(IndexReader reader, System.String field);
 		
-		/// <summary>Checks the internal cache for an appropriate entry, and if none
-		/// is found, reads the term values in <code>Field</code> and returns an array
-		/// of size <code>reader.maxDoc()</code> containing the value each document
-		/// has in the given Field.
+		/// <summary>Checks the internal cache for an appropriate entry, and if
+		/// none is found, reads the terms in <code>field</code> as floats and returns an array
+		/// of size <code>reader.maxDoc()</code> of the value each document
+		/// has in the given field.
 		/// </summary>
-		/// <param name="reader"> Used to get Field values.
+		/// <param name="reader"> Used to get field values.
 		/// </param>
-		/// <param name="Field">  Which Field contains the strings.
+		/// <param name="field">  Which field contains the floats.
 		/// </param>
-		/// <returns> The values in the given Field for each document.
+		/// <param name="parser"> Computes float for string values.
+		/// </param>
+		/// <returns> The values in the given field for each document.
+		/// </returns>
+		/// <throws>  IOException  If any error occurs. </throws>
+		float[] GetFloats(IndexReader reader, System.String field, FloatParser parser);
+		
+		/// <summary>Checks the internal cache for an appropriate entry, and if none
+		/// is found, reads the term values in <code>field</code> and returns an array
+		/// of size <code>reader.maxDoc()</code> containing the value each document
+		/// has in the given field.
+		/// </summary>
+		/// <param name="reader"> Used to get field values.
+		/// </param>
+		/// <param name="field">  Which field contains the strings.
+		/// </param>
+		/// <returns> The values in the given field for each document.
 		/// </returns>
 		/// <throws>  IOException  If any error occurs. </throws>
 		System.String[] GetStrings(IndexReader reader, System.String field);
 		
 		/// <summary>Checks the internal cache for an appropriate entry, and if none
-		/// is found reads the term values in <code>Field</code> and returns
+		/// is found reads the term values in <code>field</code> and returns
 		/// an array of them in natural order, along with an array telling
 		/// which element in the term array each document uses.
 		/// </summary>
-		/// <param name="reader"> Used to get Field values.
+		/// <param name="reader"> Used to get field values.
 		/// </param>
-		/// <param name="Field">  Which Field contains the strings.
+		/// <param name="field">  Which field contains the strings.
 		/// </param>
 		/// <returns> Array of terms and index into the array for each document.
 		/// </returns>
@@ -126,15 +156,15 @@ namespace Lucene.Net.Search
 		StringIndex GetStringIndex(IndexReader reader, System.String field);
 		
 		/// <summary>Checks the internal cache for an appropriate entry, and if
-		/// none is found reads <code>Field</code> to see if it contains integers, floats
+		/// none is found reads <code>field</code> to see if it contains integers, floats
 		/// or strings, and then calls one of the other methods in this class to get the
 		/// values.  For string values, a StringIndex is returned.  After
 		/// calling this method, there is an entry in the cache for both
 		/// type <code>AUTO</code> and the actual found type.
 		/// </summary>
-		/// <param name="reader"> Used to get Field values.
+		/// <param name="reader"> Used to get field values.
 		/// </param>
-		/// <param name="Field">  Which Field contains the values.
+		/// <param name="field">  Which field contains the values.
 		/// </param>
 		/// <returns> int[], float[] or StringIndex.
 		/// </returns>
@@ -142,14 +172,14 @@ namespace Lucene.Net.Search
 		System.Object GetAuto(IndexReader reader, System.String field);
 		
 		/// <summary>Checks the internal cache for an appropriate entry, and if none
-		/// is found reads the terms out of <code>Field</code> and calls the given SortComparator
+		/// is found reads the terms out of <code>field</code> and calls the given SortComparator
 		/// to get the sort values.  A hit in the cache will happen if <code>reader</code>,
-		/// <code>Field</code>, and <code>comparator</code> are the same (using <code>equals()</code>)
+		/// <code>field</code>, and <code>comparator</code> are the same (using <code>equals()</code>)
 		/// as a previous call to this method.
 		/// </summary>
-		/// <param name="reader"> Used to get Field values.
+		/// <param name="reader"> Used to get field values.
 		/// </param>
-		/// <param name="Field">  Which Field contains the values.
+		/// <param name="field">  Which field contains the values.
 		/// </param>
 		/// <param name="comparator">Used to convert terms into something to sort by.
 		/// </param>
@@ -158,4 +188,22 @@ namespace Lucene.Net.Search
 		/// <throws>  IOException  If any error occurs. </throws>
 		System.IComparable[] GetCustom(IndexReader reader, System.String field, SortComparator comparator);
 	}
+	
+	/// <summary>Interface to parse ints from document fields.</summary>
+	/// <seealso cref="String, IntParser)">
+	/// </seealso>
+	public interface IntParser
+	{
+		/// <summary>Return an integer representation of this field's value. </summary>
+		int ParseInt(System.String string_Renamed);
+	}
+	
+	/// <summary>Interface to parse floats from document fields.</summary>
+	/// <seealso cref="String, FloatParser)">
+	/// </seealso>
+	public interface FloatParser
+	{
+		/// <summary>Return an float representation of this field's value. </summary>
+		float ParseFloat(System.String string_Renamed);
+	}
 }
\ No newline at end of file
