@@ -1,9 +1,10 @@
 /*
- * Copyright 2004 The Apache Software Foundation
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  * http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -15,13 +16,25 @@
  */
 
 using System;
+
 using IndexFileNameFilter = Lucene.Net.Index.IndexFileNameFilter;
-using Mono.Unix.Native;
+// Used only for WRITE_LOCK_NAME in deprecated create=true case:
+using IndexWriter = Lucene.Net.Index.IndexWriter;
 
 namespace Lucene.Net.Store
 {
 	
 	/// <summary> Straightforward implementation of {@link Directory} as a directory of files.
+	/// Locking implementation is by default the {@link SimpleFSLockFactory}, but
+	/// can be changed either by passing in a {@link LockFactory} instance to
+	/// <code>getDirectory</code>, or specifying the LockFactory class by setting
+	/// <code>Lucene.Net.Store.FSDirectoryLockFactoryClass</code> Java system
+	/// property, or by calling {@link #setLockFactory} after creating
+	/// the Directory.
+	/// <p>Directories are cached, so that, for a given canonical
+	/// path, the same FSDirectory instance will always be
+	/// returned by <code>getDirectory</code>.  This permits
+	/// synchronization on directories.</p>
 	/// 
 	/// </summary>
 	/// <seealso cref="Directory">
@@ -30,165 +43,22 @@ namespace Lucene.Net.Store
 	/// </author>
 	public class FSDirectory : Directory
 	{
-		private class AnonymousClassLock : Lock
-		{
-			public AnonymousClassLock(System.IO.FileInfo lockFile, FSDirectory enclosingInstance)
-			{
-				InitBlock(lockFile, enclosingInstance);
-			}
-			private void  InitBlock(System.IO.FileInfo lockFile, FSDirectory enclosingInstance)
-			{
-				this.lockFile = lockFile;
-				this.enclosingInstance = enclosingInstance;
-			}
-			private System.IO.FileInfo lockFile;
-			private FSDirectory enclosingInstance;
-			public FSDirectory Enclosing_Instance
-			{
-				get
-				{
-					return enclosingInstance;
-				}
-				
-			}
-			public override bool Obtain()
-			{
-				Log ("Trying to obtain lock " + lockFile.FullName);
-				if (Lucene.Net.Store.FSDirectory.disableLocks || Enclosing_Instance.InstanceDisableLock)
-					return true;
-				
-				bool tmpBool;
-				if (System.IO.File.Exists(Enclosing_Instance.lockDir.FullName))
-					tmpBool = true;
-				else
-					tmpBool = System.IO.Directory.Exists(Enclosing_Instance.lockDir.FullName);
-				if (!tmpBool)
-				{
-                    try
-                    {
-					    System.IO.Directory.CreateDirectory(Enclosing_Instance.lockDir.FullName);
-                    }
-                    catch (Exception)
-					{
-						throw new System.IO.IOException("Cannot create lock directory: " + Enclosing_Instance.lockDir);
-					}
-				}
-
-                try
-                {
-		    int fd = Mono.Unix.Native.Syscall.open (
-				lockFile.FullName,
-				Mono.Unix.Native.OpenFlags.O_RDWR | 
-				Mono.Unix.Native.OpenFlags.O_CREAT |
-				Mono.Unix.Native.OpenFlags.O_EXCL,
-				Mono.Unix.Native.FilePermissions.S_IRUSR);
-		    if (fd == -1) {
-			    Mono.Unix.Native.Errno error = Mono.Unix.Native.Stdlib.GetLastError ();
-			    if (error == Mono.Unix.Native.Errno.ENOSPC)
-				    throw new Beagle.Util.NoSpaceException ();
-			    else
-				    throw new System.IO.IOException ("Could not create lock file: "
-					    + Mono.Unix.Native.Stdlib.strerror (error));
-		    }
-
-		    // This code replaces the commented-out code below because
-		    // it ends up being much faster.  The reason for this is
-		    // that closing a UnixStream causes Syscall.fsync() to be
-		    // called, and that apparently is extremely slow!
-		    //
-		    // Time(ms) Count   P/call(ms) Method name
-		    // 1563.926      68   22.999   Mono.Unix.Native.Syscall::fsync(int)
-		    //
-		    // Since the lock file is written out very often, this time
-		    // adds up and noticably slows down indexing.
-		    IntPtr ptr = IntPtr.Zero;
-		    long ret;
-
-		    try {
-			    string s = System.Diagnostics.Process.GetCurrentProcess ().Id.ToString () + "\n";
-			    ptr = Mono.Unix.UnixMarshal.StringToHeap (s);
-
-			    do {
-				    ret = Mono.Unix.Native.Syscall.write (fd, ptr, (ulong) s.Length);
-			    } while (Mono.Unix.UnixMarshal.ShouldRetrySyscall ((int) ret));
-			    if ((int)ret == -1) {
-				    Mono.Unix.Native.Errno error = Mono.Unix.Native.Stdlib.GetLastError ();
-				    if (error == Mono.Unix.Native.Errno.ENOSPC)
-					    throw new Beagle.Util.NoSpaceException ();
-				    else
-					    Mono.Unix.UnixMarshal.ThrowExceptionForError (error);
-			    }
-		    } finally {
-			    Mono.Unix.UnixMarshal.FreeHeap (ptr);
-
-			    do {
-				    ret = Mono.Unix.Native.Syscall.close (fd);
-			    } while (Mono.Unix.UnixMarshal.ShouldRetrySyscall ((int) ret));
-			    Mono.Unix.UnixMarshal.ThrowExceptionForLastErrorIf ((int) ret);
-		    }
-
-		    //System.IO.StreamWriter w = new System.IO.StreamWriter (new Mono.Unix.UnixStream (fd, true));
-		    //w.WriteLine (System.Diagnostics.Process.GetCurrentProcess ().Id);
-		    //w.Close ();
-                    return true;
-                }
-		catch (Beagle.Util.NoSpaceException e)
-		{
-			throw e;
-		}
-                catch (Exception e)
-                {
-		    Log ("Exception in CreateNew for file:" + lockFile.FullName + ":" + e);
-                    return false;
-                }
-            }
-			public override void  Release()
-			{
-				if (Lucene.Net.Store.FSDirectory.disableLocks || Enclosing_Instance.InstanceDisableLock)
-					return ;
-
-				int fd = Mono.Unix.Native.Syscall.unlink (
-		    		    	lockFile.FullName);
-		    		if (fd == -1)
-					throw new System.IO.IOException (
-						"Could not release lock file: "
-						+ Mono.Unix.Native.Stdlib.strerror (Mono.Unix.Native.Stdlib.GetLastError ()
-					));
-
-				if (System.IO.File.Exists(lockFile.FullName)) {
-					Beagle.Util.Logger.Log.Warn ("Release didnt delete lockfile {0}.", lockFile.FullName);
-				}
-			}
-
-			public override bool IsLocked()
-			{
-				if (Lucene.Net.Store.FSDirectory.disableLocks || Enclosing_Instance.InstanceDisableLock)
-					return false;
-				bool tmpBool;
-				if (System.IO.File.Exists(lockFile.FullName))
-					tmpBool = true;
-				else
-					tmpBool = System.IO.Directory.Exists(lockFile.FullName);
-				return tmpBool;
-			}
-			
-			public override System.String ToString()
-			{
-				return lockFile.FullName;
-			}
-		}
 		
 		/// <summary>This cache of directories ensures that there is a unique Directory
 		/// instance per path, so that synchronization on the Directory can be used to
-		/// synchronize access between readers and writers.
-		/// 
-		/// This should be a WeakHashMap, so that entries can be GC'd, but that would
-		/// require Java 1.2.  Instead we use refcounts...
+		/// synchronize access between readers and writers.  We use
+		/// refcounts to ensure when the last use of an FSDirectory
+		/// instance for a given canonical path is closed, we remove the
+		/// instance from the cache.  See LUCENE-776
+		/// for some relevant discussion.
 		/// </summary>
 		private static readonly System.Collections.Hashtable DIRECTORIES = System.Collections.Hashtable.Synchronized(new System.Collections.Hashtable());
 		
 		private static bool disableLocks = false;
 		
+		// TODO: should this move up to the Directory base class?  Also: should we
+		// make a per-instance (in addition to the static "default") version?
+		
 		/// <summary> Set whether Lucene's use of lock files is disabled. By default, 
 		/// lock files are enabled. They should only be disabled if the index
 		/// is on a read-only medium like a CD-ROM.
@@ -205,17 +75,21 @@ namespace Lucene.Net.Store
 		{
 			return FSDirectory.disableLocks;
 		}
-
-		private bool instance_disable_lock = false;
-		public bool InstanceDisableLock {
-			get { return instance_disable_lock; }
-			set { instance_disable_lock = value; }
-		}
-
+		
 		/// <summary> Directory specified by <code>Lucene.Net.lockDir</code>
-		/// or <code>java.io.tmpdir</code> system property
+		/// or <code>java.io.tmpdir</code> system property.
 		/// </summary>
-		public static readonly System.String LOCK_DIR = SupportClass.AppSettings.Get("Lucene.Net.lockDir", System.IO.Path.GetTempPath());
+		/// <deprecated> As of 2.1, <code>LOCK_DIR</code> is unused
+		/// because the write.lock is now stored by default in the
+		/// index directory.  If you really want to store locks
+		/// elsewhere you can create your own {@link
+		/// SimpleFSLockFactory} (or {@link NativeFSLockFactory},
+		/// etc.) passing in your preferred lock directory.  Then,
+		/// pass this <code>LockFactory</code> instance to one of
+		/// the <code>getDirectory</code> methods that take a
+		/// <code>lockFactory</code> (for example, {@link #GetDirectory(String, LockFactory)}).
+		/// </deprecated>
+		/// REMOVED - D Bera
 		
 		/// <summary>The default class which implements filesystem-based directories. </summary>
 		private static System.Type IMPL;
@@ -225,108 +99,78 @@ namespace Lucene.Net.Store
 		/// <summary>A buffer optionally used in renameTo method </summary>
 		private byte[] buffer = null;
 		
-		/// <summary>Returns the directory instance for the named location.
-		/// 
-		/// <p>Directories are cached, so that, for a given canonical path, the same
-		/// FSDirectory instance will always be returned.  This permits
-		/// synchronization on directories.
-		/// 
-		/// </summary>
+		/// <summary>Returns the directory instance for the named location.</summary>
 		/// <param name="path">the path to the directory.
 		/// </param>
-		/// <param name="create">if true, create, or erase any existing contents.
-		/// </param>
 		/// <returns> the FSDirectory for the named file.  
 		/// </returns>
-		public static FSDirectory GetDirectory(System.String path, bool create)
+		public static FSDirectory GetDirectory(System.String path)
 		{
-			return GetDirectory(new System.IO.FileInfo(path), null, create, false);
+			return GetDirectory(new System.IO.FileInfo(path), null);
 		}
 		
-		/// <summary>Returns the directory instance for the named location.
-		/// 
-		/// <p>Directories are cached, so that, for a given canonical path, the same
-		/// FSDirectory instance will always be returned.  This permits
-		/// synchronization on directories.
-		/// 
-		/// </summary>
-		/// <param name="file">the path to the directory.
-		/// </param>
-		/// <param name="create">if true, create, or erase any existing contents.
-		/// </param>
-		/// <returns> the FSDirectory for the named file.  
-		/// </returns>
-		public static FSDirectory GetDirectory(System.IO.FileInfo file, bool create)
-		{
-			return GetDirectory(file, null, create, false);
-		}
-
-		/// <summary>Returns the directory instance for the named location.
-		/// 
-		/// <p>Directories are cached, so that, for a given canonical path, the same
-		/// FSDirectory instance will always be returned.  This permits
-		/// synchronization on directories.
-		/// 
-		/// </summary>
+		/// <summary>Returns the directory instance for the named location.</summary>
 		/// <param name="path">the path to the directory.
 		/// </param>
-		/// <param name="tmpDir">the path to use as lock directory.
-		/// </param>
-		/// <param name="create">if true, create, or erase any existing contents.
+		/// <param name="lockFactory">instance of {@link LockFactory} providing the
+		/// locking implementation.
 		/// </param>
 		/// <returns> the FSDirectory for the named file.  
 		/// </returns>
-		public static FSDirectory GetDirectory(System.String path, System.String tmpDir, bool create)
+		public static FSDirectory GetDirectory(System.String path, LockFactory lockFactory)
 		{
-			return GetDirectory(new System.IO.FileInfo(path), tmpDir, create, false);
+			return GetDirectory(new System.IO.FileInfo(path), lockFactory);
 		}
-
-		/// <summary>Returns the directory instance for the named location.
-		/// 
-		/// <p>Directories are cached, so that, for a given canonical path, the same
-		/// FSDirectory instance will always be returned.  This permits
-		/// synchronization on directories.
-		/// 
-		/// </summary>
-		/// <param name="path">the path to the directory.
-		/// </param>
-		/// <param name="tmpDir">the path to use as lock directory.
-		/// </param>
-		/// <param name="create">if true, create, or erase any existing contents.
-		/// </param>
-		/// <param name="disable_locks">if true, disable internal locking
-		/// </param>
-		/// <returns> the FSDirectory for the named file.  
-		/// </returns>
-		public static FSDirectory GetDirectory(System.String path, System.String tmpDir, bool create, bool disable_locks)
-		{
-			return GetDirectory(new System.IO.FileInfo(path), tmpDir, create, disable_locks);
-		}
-
-		/// <summary>Returns the directory instance for the named location.
-		/// 
-		/// <p>Directories are cached, so that, for a given canonical path, the same
-		/// FSDirectory instance will always be returned.  This permits
-		/// synchronization on directories.
-		/// 
-		/// </summary>
+		
+		/// <summary>Returns the directory instance for the named location.</summary>
 		/// <param name="file">the path to the directory.
 		/// </param>
-		/// <param name="tmpDir">the path to use as lock directory.
+		/// <returns> the FSDirectory for the named file.  
+		/// </returns>
+		public static FSDirectory GetDirectory(System.IO.FileInfo file)
+		{
+			return GetDirectory(file, null);
+		}
+		
+		/// <summary>Returns the directory instance for the named location.</summary>
+		/// <param name="file">the path to the directory.
 		/// </param>
-		/// <param name="create">if true, create, or erase any existing contents.
-		/// </param>
-		/// <param name="disable_locks">if true, disable internal locking
+		/// <param name="lockFactory">instance of {@link LockFactory} providing the
+		/// locking implementation.
 		/// </param>
 		/// <returns> the FSDirectory for the named file.  
 		/// </returns>
-		public static FSDirectory GetDirectory(System.IO.FileInfo file, System.String tmpdir, bool create, bool disable_locks)
+		public static FSDirectory GetDirectory(System.IO.FileInfo file, LockFactory lockFactory)
 		{
-			file = new System.IO.FileInfo(file.FullName);
+			bool tmpBool;
+			if (System.IO.File.Exists(file.FullName))
+				tmpBool = true;
+			else
+				tmpBool = System.IO.Directory.Exists(file.FullName);
+			if (tmpBool && !System.IO.Directory.Exists(file.FullName))
+				throw new System.IO.IOException(file + " not a directory");
+			
+			bool tmpBool2;
+			if (System.IO.File.Exists(file.FullName))
+				tmpBool2 = true;
+			else
+				tmpBool2 = System.IO.Directory.Exists(file.FullName);
+			if (!tmpBool2)
+			{
+                try
+                {
+                    System.IO.Directory.CreateDirectory(file.FullName);
+                }
+                catch
+                {
+                    throw new System.IO.IOException("Cannot create directory: " + file);
+                }
+			}
+			
 			FSDirectory dir;
 			lock (DIRECTORIES.SyncRoot)
 			{
-				dir = (FSDirectory) DIRECTORIES[file];
+				dir = (FSDirectory) DIRECTORIES[file.FullName];
 				if (dir == null)
 				{
 					try
@@ -335,14 +179,19 @@ namespace Lucene.Net.Store
 					}
 					catch (System.Exception e)
 					{
-						throw new System.SystemException("cannot load FSDirectory class: " + e.ToString());
+						throw new System.SystemException("cannot load FSDirectory class: " + e.ToString(), e);
 					}
-					dir.Init(file, tmpdir, create, disable_locks);
-					DIRECTORIES[file] = dir;
+					dir.Init(file, lockFactory);
+					DIRECTORIES[file.FullName] = dir;
 				}
-				else if (create)
+				else
 				{
-					dir.Create();
+					// Catch the case where a Directory is pulled from the cache, but has a
+					// different LockFactory instance.
+					if (lockFactory != null && lockFactory != dir.GetLockFactory())
+					{
+						throw new System.IO.IOException("Directory was previously created with a different LockFactory instance; please pass null as the lockFactory instance and use setLockFactory to change it");
+					}
 				}
 			}
 			lock (dir)
@@ -352,177 +201,172 @@ namespace Lucene.Net.Store
 			return dir;
 		}
 		
-		private System.IO.FileInfo directory = null;
-		private int refCount;
-		private System.IO.FileInfo lockDir;
 		
-		public FSDirectory() // protected internal FSDirectory() // {{Aroush}} this shouldn't be 'public' but if it's not the line 'System.Activator.CreateInstance(IMPL);' in function GetDirectory() will fail.
+		/// <summary>Returns the directory instance for the named location.
+		/// 
+		/// </summary>
+		/// <deprecated> Use IndexWriter's create flag, instead, to
+		/// create a new index.
+		/// 
+		/// </deprecated>
+		/// <param name="path">the path to the directory.
+		/// </param>
+		/// <param name="create">if true, create, or erase any existing contents.
+		/// </param>
+		/// <returns> the FSDirectory for the named file.  
+		/// </returns>
+		public static FSDirectory GetDirectory(System.String path, bool create)
 		{
+			return GetDirectory(new System.IO.FileInfo(path), create);
 		}
 		
-		// permit subclassing
-		
-	        private void  Init(System.IO.FileInfo path, System.String tmpDir, bool create, bool disable_locks)
+		/// <summary>Returns the directory instance for the named location.
+		/// 
+		/// </summary>
+		/// <deprecated> Use IndexWriter's create flag, instead, to
+		/// create a new index.
+		/// 
+		/// </deprecated>
+		/// <param name="file">the path to the directory.
+		/// </param>
+		/// <param name="create">if true, create, or erase any existing contents.
+		/// </param>
+		/// <returns> the FSDirectory for the named file.  
+		/// </returns>
+		public static FSDirectory GetDirectory(System.IO.FileInfo file, bool create)
 		{
-			directory = path;
-			this.instance_disable_lock = disable_locks;
-
-
-			if (tmpDir != null)
-			{
-				lockDir = new System.IO.FileInfo(tmpDir);
-			}
-			else if (LOCK_DIR == null)
-			{
-				lockDir = directory;
-			}
-			else
-			{
-				lockDir = new System.IO.FileInfo(LOCK_DIR);
-			}
-			// Ensure that lockDir exists and is a directory.
-			bool tmpBool;
-			if (System.IO.File.Exists(lockDir.FullName))
-				tmpBool = true;
-			else
-				tmpBool = System.IO.Directory.Exists(lockDir.FullName);
-			if (!tmpBool)
-			{
-                try
-                {
-                    System.IO.Directory.CreateDirectory(lockDir.FullName);
-                }
-                catch (Exception)
-                {
-                    throw new System.IO.IOException("Cannot create directory: " + lockDir);
-                }
-			}
-			else if (!System.IO.Directory.Exists(lockDir.FullName))
-			{
-				throw new System.IO.IOException("Found regular file where directory expected: " + lockDir);
-			}
+			FSDirectory dir = GetDirectory(file, null);
+			
+			// This is now deprecated (creation should only be done
+			// by IndexWriter):
 			if (create)
 			{
-				Create();
+				dir.Create();
 			}
 			
-			if (!System.IO.Directory.Exists(directory.FullName))
-				throw new System.IO.IOException(path + " not a directory");
+			return dir;
 		}
 		
 		private void  Create()
 		{
-			lock (this)
+			bool tmpBool;
+			if (System.IO.File.Exists(directory.FullName))
+				tmpBool = true;
+			else
+				tmpBool = System.IO.Directory.Exists(directory.FullName);
+			if (tmpBool)
 			{
-				bool tmpBool;
-				if (System.IO.File.Exists(directory.FullName))
-					tmpBool = true;
-				else
-					tmpBool = System.IO.Directory.Exists(directory.FullName);
-				if (!tmpBool)
-				{
-                    try
-                    {
-                        System.IO.Directory.CreateDirectory(directory.FullName);
-                    }
-                    catch (Exception)
-                    {
-                        throw new System.IO.IOException("Cannot create directory: " + directory);
-                    }
-				}
-				
-                try
-                {
-                    System.IO.Directory.Exists(directory.FullName);
-                }
-                catch (Exception)
-                {
-                    throw new System.IO.IOException(directory + " not a directory");
-                }
-				
-                System.String[] files = System.IO.Directory.GetFileSystemEntries(directory.FullName); // clear old files    // {{Aroush-1.9}} we want the line below, not this one; how do we make the line below work in C#?!
-                //// System.String[] files = System.IO.Directory.GetFileSystemEntries(new IndexFileNameFilter()); // clear old files 
+                System.String[] files = SupportClass.FileSupport.GetLuceneIndexFiles(directory.FullName, IndexFileNameFilter.GetFilter());
+				if (files == null)
+					throw new System.IO.IOException("Cannot read directory " + directory.FullName);
 				for (int i = 0; i < files.Length; i++)
 				{
-					System.IO.FileInfo file = new System.IO.FileInfo(System.IO.Path.Combine(directory.FullName, files[i]));
+					string file = System.IO.Path.Combine(directory.FullName, files[i]);
 					bool tmpBool2;
-					if (System.IO.File.Exists(file.FullName))
+					if (System.IO.File.Exists(file))
 					{
-						System.IO.File.Delete(file.FullName);
+						System.IO.File.Delete(file);
 						tmpBool2 = true;
 					}
-					else if (System.IO.Directory.Exists(file.FullName))
+					else if (System.IO.Directory.Exists(file))
 					{
-						System.IO.Directory.Delete(file.FullName);
+						System.IO.Directory.Delete(file);
 						tmpBool2 = true;
 					}
 					else
 						tmpBool2 = false;
 					if (!tmpBool2)
-						throw new System.IO.IOException("Cannot delete " + files[i]);
+						throw new System.IO.IOException("Cannot delete " + file);
 				}
+			}
+			lockFactory.ClearLock(IndexWriter.WRITE_LOCK_NAME);
+		}
+		
+		private System.IO.FileInfo directory = null;
+		private int refCount;
+		
+		public FSDirectory()
+		{
+		}
+		
+		// permit subclassing
+		
+		private void  Init(System.IO.FileInfo path, LockFactory lockFactory)
+		{
+			
+			// Set up lockFactory with cascaded defaults: if an instance was passed in,
+			// use that; else if locks are disabled, use NoLockFactory; else if the
+			// system property Lucene.Net.Store.FSDirectoryLockFactoryClass is set,
+			// instantiate that; else, use SimpleFSLockFactory:
+			
+			directory = path;
+			
+			bool doClearLockID = false;
+			
+			if (lockFactory == null)
+			{
 				
-				System.String lockPrefix = GetLockPrefix().ToString(); // clear old locks
-				files = System.IO.Directory.GetFileSystemEntries(lockDir.FullName);
-				if (files == null)
-					throw new System.IO.IOException("Cannot read lock directory " + lockDir.FullName);
-				for (int i = 0; i < files.Length; i++)
+				if (disableLocks)
 				{
-					if (!files[i].StartsWith(lockPrefix))
-						continue;
-					System.IO.FileInfo lockFile = new System.IO.FileInfo(System.IO.Path.Combine(lockDir.FullName, files[i]));
-					bool tmpBool3;
-					if (System.IO.File.Exists(lockFile.FullName))
-					{
-						System.IO.File.Delete(lockFile.FullName);
-						tmpBool3 = true;
-					}
-					else if (System.IO.Directory.Exists(lockFile.FullName))
-					{
-						System.IO.Directory.Delete(lockFile.FullName);
-						tmpBool3 = true;
-					}
-					else
-						tmpBool3 = false;
-					if (!tmpBool3)
-						throw new System.IO.IOException("Cannot delete " + files[i]);
+					// Locks are disabled:
+					lockFactory = NoLockFactory.GetNoLockFactory();
 				}
+				else
+				{
+					// Our default lock is SimpleFSLockFactory;
+					// default lockDir is our index directory:
+					lockFactory = new SimpleFSLockFactory(path);
+					doClearLockID = true;
+				}
+			}
+			
+			SetLockFactory(lockFactory);
+			
+			if (doClearLockID)
+			{
+				// Clear the prefix because write.lock will be
+				// stored in our directory:
+				lockFactory.SetLockPrefix(null);
 			}
 		}
 		
-		/// <summary>Returns an array of strings, one for each file in the directory. </summary>
+		/// <summary>Returns an array of strings, one for each Lucene index file in the directory. </summary>
 		public override System.String[] List()
 		{
-			return System.IO.Directory.GetFileSystemEntries(directory.FullName);
+            System.String[] files = SupportClass.FileSupport.GetLuceneIndexFiles(directory.FullName, IndexFileNameFilter.GetFilter());
+            for (int i = 0; i < files.Length; i++)
+            {
+		files[i] = System.IO.Path.GetFileName(files[i]);
+            }
+			return files;
 		}
 		
 		/// <summary>Returns true iff a file with the given name exists. </summary>
 		public override bool FileExists(System.String name)
 		{
-			string path = System.IO.Path.Combine (directory.FullName, name);
-
-			return System.IO.File.Exists (path) || System.IO.Directory.Exists (path);
+			string file = System.IO.Path.Combine(directory.FullName, name);
+			return System.IO.File.Exists(file) || System.IO.Directory.Exists(file);
 		}
 		
 		/// <summary>Returns the time the named file was last modified. </summary>
 		public override long FileModified(System.String name)
 		{
-			System.IO.FileInfo file = new System.IO.FileInfo(System.IO.Path.Combine(directory.FullName, name));
-			return (file.LastWriteTimeUtc.Ticks);
+			string file = System.IO.Path.Combine(directory.FullName, name);
+			return (System.IO.File.GetLastWriteTime(file).Ticks);
 		}
 		
 		/// <summary>Returns the time the named file was last modified. </summary>
 		public static long FileModified(System.IO.FileInfo directory, System.String name)
 		{
-			System.IO.FileInfo file = new System.IO.FileInfo(System.IO.Path.Combine(directory.FullName, name));
-			return (file.LastWriteTimeUtc.Ticks);
+			string file = System.IO.Path.Combine(directory.FullName, name);
+			return (System.IO.File.GetLastWriteTime(file).Ticks);
 		}
 		
 		/// <summary>Set the modified time of an existing file to now. </summary>
 		public override void  TouchFile(System.String name)
 		{
-			System.IO.FileInfo file = new System.IO.FileInfo(System.IO.Path.Combine(directory.FullName, name));
-            file.LastWriteTime = System.DateTime.UtcNow;
+			string file = System.IO.Path.Combine(directory.FullName, name);
+			System.IO.File.SetLastWriteTimeUtc(file, System.DateTime.UtcNow);
 		}
 		
 		/// <summary>Returns the length in bytes of a file in the directory. </summary>
@@ -535,16 +379,16 @@ namespace Lucene.Net.Store
 		/// <summary>Removes an existing file in the directory. </summary>
 		public override void  DeleteFile(System.String name)
 		{
-			System.IO.FileInfo file = new System.IO.FileInfo(System.IO.Path.Combine(directory.FullName, name));
+			string file = System.IO.Path.Combine(directory.FullName, name);
 			bool tmpBool;
-			if (System.IO.File.Exists(file.FullName))
+			if (System.IO.File.Exists(file))
 			{
-				System.IO.File.Delete(file.FullName);
+				System.IO.File.Delete(file);
 				tmpBool = true;
 			}
-			else if (System.IO.Directory.Exists(file.FullName))
+			else if (System.IO.Directory.Exists(file))
 			{
-				System.IO.Directory.Delete(file.FullName);
+				System.IO.Directory.Delete(file);
 				tmpBool = true;
 			}
 			else
@@ -553,7 +397,11 @@ namespace Lucene.Net.Store
 				throw new System.IO.IOException("Cannot delete " + file);
 		}
 		
-		/// <summary>Renames an existing file in the directory. </summary>
+		/// <summary>Renames an existing file in the directory. 
+		/// Warning: This is not atomic.
+		/// </summary>
+		/// <deprecated> 
+		/// </deprecated>
 		public override void  RenameFile(System.String from, System.String to)
 		{
 			lock (this)
@@ -596,7 +444,7 @@ namespace Lucene.Net.Store
                 {
                     old.MoveTo(nu.FullName);
                 }
-                catch (System.Exception)
+                catch
 				{
 					System.IO.Stream in_Renamed = null;
 					System.IO.Stream out_Renamed = null;
@@ -611,13 +459,13 @@ namespace Lucene.Net.Store
 						{
 							buffer = new byte[1024];
 						}
-                        int len; 
-                        while ((len = in_Renamed.Read(buffer, 0, buffer.Length)) > 0) 
-                        { 
-                            out_Renamed.Write(buffer, 0, len); 
-                        }
+						int len;
+						while ((len = in_Renamed.Read(buffer, 0, buffer.Length)) >= 0)
+						{
+							out_Renamed.Write(buffer, 0, len);
+						}
 						
-                        // delete the old file.
+						// delete the old file.
 						bool tmpBool3;
 						if (System.IO.File.Exists(old.FullName))
 						{
@@ -635,30 +483,37 @@ namespace Lucene.Net.Store
 					}
 					catch (System.IO.IOException ioe)
 					{
-						throw new System.IO.IOException("Cannot rename " + old + " to " + nu);
+						System.IO.IOException newExc = new System.IO.IOException("Cannot rename " + old + " to " + nu, ioe);
+						throw newExc;
 					}
 					finally
 					{
-						if (in_Renamed != null)
+						try
 						{
-							try
+							if (in_Renamed != null)
 							{
-								in_Renamed.Close();
-							}
-							catch (System.IO.IOException e)
-							{
-								throw new System.SystemException("Cannot close input stream: " + e.ToString());
+								try
+								{
+									in_Renamed.Close();
+								}
+								catch (System.IO.IOException e)
+								{
+									throw new System.SystemException("Cannot close input stream: " + e.ToString(), e);
+								}
 							}
 						}
-						if (out_Renamed != null)
+						finally
 						{
-							try
+							if (out_Renamed != null)
 							{
-								out_Renamed.Close();
-							}
-							catch (System.IO.IOException e)
-							{
-								throw new System.SystemException("Cannot close output stream: " + e.ToString());
+								try
+								{
+									out_Renamed.Close();
+								}
+								catch (System.IO.IOException e)
+								{
+									throw new System.SystemException("Cannot close output stream: " + e.ToString(), e);
+								}
 							}
 						}
 					}
@@ -671,6 +526,7 @@ namespace Lucene.Net.Store
 		/// </summary>
 		public override IndexOutput CreateOutput(System.String name)
 		{
+			
 			System.IO.FileInfo file = new System.IO.FileInfo(System.IO.Path.Combine(directory.FullName, name));
 			bool tmpBool;
 			if (System.IO.File.Exists(file.FullName))
@@ -703,44 +559,11 @@ namespace Lucene.Net.Store
 			return new FSIndexInput(new System.IO.FileInfo(System.IO.Path.Combine(directory.FullName, name)));
 		}
 		
-		static public Beagle.Util.Logger Logger = null;
-		//static public Beagle.Util.Logger Logger = Beagle.Util.Logger.Log;
-		static public void Log (string format, params object[] args)
-		{
-			if (Logger != null)
-				Logger.Debug (format, args);
-		}
-
-		static public void Log (Exception e)
-		{
-			if (Logger != null)
-				Logger.Debug (e);
-		}
-		
 		/// <summary> So we can do some byte-to-hexchar conversion below</summary>
 		private static readonly char[] HEX_DIGITS = new char[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
 		
-		/// <summary>Constructs a {@link Lock} with the specified name.  Locks are implemented
-		/// with {@link File#createNewFile()}.
-		/// 
-		/// </summary>
-		/// <param name="name">the name of the lock file
-		/// </param>
-		/// <returns> an instance of <code>Lock</code> holding the lock
-		/// </returns>
-		public override Lock MakeLock(System.String name)
-		{
-			System.Text.StringBuilder buf = GetLockPrefix();
-			buf.Append("-");
-			buf.Append(name);
-			
-			// create a lock file
-			System.IO.FileInfo lockFile = new System.IO.FileInfo(System.IO.Path.Combine(lockDir.FullName, buf.ToString()));
-			
-			return new AnonymousClassLock(lockFile, this);
-		}
 		
-		private System.Text.StringBuilder GetLockPrefix()
+		public override System.String GetLockID()
 		{
 			System.String dirName; // name to be hashed
 			try
@@ -749,7 +572,7 @@ namespace Lucene.Net.Store
 			}
 			catch (System.IO.IOException e)
 			{
-				throw new System.SystemException(e.ToString());
+				throw new System.SystemException(e.ToString(), e);
 			}
 			
 			byte[] digest;
@@ -766,7 +589,7 @@ namespace Lucene.Net.Store
 				buf.Append(HEX_DIGITS[b & 0xf]);
 			}
 			
-			return buf;
+			return buf.ToString();
 		}
 		
 		/// <summary>Closes the store to future operations. </summary>
@@ -778,7 +601,7 @@ namespace Lucene.Net.Store
 				{
 					lock (DIRECTORIES.SyncRoot)
 					{
-						DIRECTORIES.Remove(directory);
+						DIRECTORIES.Remove(directory.FullName);
 					}
 				}
 			}
@@ -800,7 +623,7 @@ namespace Lucene.Net.Store
 			{
 				try
 				{
-					System.String name = SupportClass.AppSettings.Get("Lucene.Net.FSDirectory.class", typeof(FSDirectory).FullName);
+					System.String name = typeof(FSDirectory).FullName;
 					IMPL = System.Type.GetType(name);
 				}
 				catch (System.Security.SecurityException)
@@ -811,12 +634,12 @@ namespace Lucene.Net.Store
 					}
 					catch (System.Exception e)
 					{
-						throw new System.SystemException("cannot load default FSDirectory class: " + e.ToString());
+						throw new System.SystemException("cannot load default FSDirectory class: " + e.ToString(), e);
 					}
 				}
                 catch (System.Exception e)
                 {
-                    throw new System.SystemException("cannot load FSDirectory class: " + e.ToString());
+                    throw new System.SystemException("cannot load FSDirectory class: " + e.ToString(), e);
                 }
             }
 			{
@@ -826,7 +649,7 @@ namespace Lucene.Net.Store
 				}
 				catch (System.Exception e)
 				{
-					throw new System.SystemException(e.ToString());
+					throw new System.SystemException(e.ToString(), e);
 				}
 			}
 		}
@@ -835,42 +658,61 @@ namespace Lucene.Net.Store
 	
 	public class FSIndexInput : BufferedIndexInput, System.ICloneable
 	{
+		/// <summary>Method used for testing. Returns true if the underlying
+		/// file descriptor is valid.
+		/// </summary>
+		public virtual bool IsFDValid()
+		{
+			return (file.BaseStream != null);
+		}
+		
 		private class Descriptor : System.IO.BinaryReader
 		{
-			private void  InitBlock(FSIndexInput enclosingInstance)
-			{
-				this.enclosingInstance = enclosingInstance;
-			}
-			private FSIndexInput enclosingInstance;
-			public FSIndexInput Enclosing_Instance
-			{
-				get
-				{
-					return enclosingInstance;
-				}
-				
-			}
-			public long position;
-			public Descriptor(FSIndexInput enclosingInstance, System.IO.FileInfo file, System.IO.FileAccess mode) 
+			// remember if the file is open, so that we don't try to close it
+			// more than once
+			private bool isOpen;
+			internal long position;
+			internal long length;
+			
+            public Descriptor(FSIndexInput enclosingInstance, System.IO.FileInfo file, System.IO.FileAccess mode) 
                 : base(new System.IO.FileStream(file.FullName, System.IO.FileMode.Open, mode, System.IO.FileShare.ReadWrite))
+            {
+				isOpen = true;
+                length = file.Length;
+			}
+			
+			public override void  Close()
 			{
-				InitBlock(enclosingInstance);
+				if (isOpen)
+				{
+					isOpen = false;
+					base.Close();
+				}
+			}
+			
+			~Descriptor()
+			{
+				try
+				{
+					Close();
+				}
+				finally
+				{
+				}
 			}
 		}
 		
-		private Descriptor file = null;
+		private Descriptor file;
 		internal bool isClone;
-		private long length;
 		
-        public bool IsClone
+        public bool isClone_ForNUnitTest
         {
-            get { return (isClone); }
+            get { return isClone; }
         }
 
 		public FSIndexInput(System.IO.FileInfo path)
 		{
 			file = new Descriptor(this, path, System.IO.FileAccess.Read);
-			length = file.BaseStream.Length;
 		}
 		
 		/// <summary>IndexInput methods </summary>
@@ -899,7 +741,8 @@ namespace Lucene.Net.Store
 		
 		public override void  Close()
 		{
-			if (!isClone && file != null)
+			// only close the file if this is not a clone
+			if (!isClone)
 				file.Close();
             System.GC.SuppressFinalize(this);
 		}
@@ -910,12 +753,7 @@ namespace Lucene.Net.Store
 		
 		public override long Length()
 		{
-			return length;
-		}
-		
-		~FSIndexInput()
-		{
-			Close(); // close the file
+			return file.length;
 		}
 		
 		public override System.Object Clone()
@@ -924,16 +762,6 @@ namespace Lucene.Net.Store
 			clone.isClone = true;
 			return clone;
 		}
-		
-		/// <summary>Method used for testing. Returns true if the underlying
-		/// file descriptor is valid.
-		/// </summary>
-		public /*internal*/ virtual bool IsFDValid()
-		{
-            if (file.BaseStream == null)
-                return false;
-			return file.BaseStream.CanRead;
-		}
 	}
 	
 	
@@ -941,9 +769,14 @@ namespace Lucene.Net.Store
 	{
 		internal System.IO.BinaryWriter file = null;
 		
+		// remember if the file is open, so that we don't try to close it
+		// more than once
+		private bool isOpen;
+		
 		public FSIndexOutput(System.IO.FileInfo path)
 		{
 			file = new System.IO.BinaryWriter(new System.IO.FileStream(path.FullName, System.IO.FileMode.OpenOrCreate, System.IO.FileAccess.Write, System.IO.FileShare.ReadWrite));
+			isOpen = true;
 		}
 		
 		/// <summary>output methods: </summary>
@@ -953,9 +786,14 @@ namespace Lucene.Net.Store
 		}
 		public override void  Close()
 		{
-			base.Close();
-			file.Close();
-			System.GC.SuppressFinalize(this);
+			// only close the file if it has not been closed yet
+			if (isOpen)
+			{
+				base.Close();
+				file.Close();
+				isOpen = false;
+                System.GC.SuppressFinalize(this);
+			}
 		}
 		
 		/// <summary>Random-access methods </summary>
@@ -968,10 +806,5 @@ namespace Lucene.Net.Store
 		{
 			return file.BaseStream.Length;
 		}
-		
-		~FSIndexOutput()
-		{
-			file.Close(); // close the file
-		}
 	}
 }
