@@ -74,7 +74,6 @@ namespace Beagle.Daemon {
 		private LuceneQueryingDriver driver;
 		private IIndexer indexer = null;
 
-		private LuceneQueryingDriver.UriFilter our_uri_filter;
 		private LuceneCommon.HitFilter our_hit_filter;
 		private LuceneCommon.QueryPartHook backend_query_part_hook;
 		private Scheduler.Task our_final_flush_task = null;
@@ -98,13 +97,13 @@ namespace Beagle.Daemon {
 			this.read_only_mode = read_only_mode;
 
 			driver = BuildLuceneQueryingDriver (this.index_name, this.minor_version, this.read_only_mode);
-			our_uri_filter = new LuceneQueryingDriver.UriFilter (this.HitIsValid);
 			our_hit_filter = new LuceneCommon.HitFilter (this.HitFilter);
 			backend_query_part_hook = new LuceneCommon.QueryPartHook (this.QueryPartHook);
 
 			// If the queryable is in read-only more, don't 
 			// instantiate an indexer for it.
-			if (read_only_mode)
+			// FIXME: --indexing-delay -1 is a hack for --read-only; need to fix this
+			if (read_only_mode || QueryDriver.IndexingDelay < 0)
 				return;
 
 			indexer = LocalIndexerHook ();
@@ -180,11 +179,6 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////
 
-		virtual protected bool HitIsValid (Uri uri)
-		{
-			return true;
-		}
-
 		virtual protected bool HitFilter (Hit hit)
 		{
 			return true;
@@ -280,6 +274,17 @@ namespace Beagle.Daemon {
 			public ICollection RemovedUris;
 		}
 
+#if ENABLE_RDF_ADAPTER
+		protected virtual TextCache TextCache {
+			get { return TextCache.UserCache; }
+		}
+
+		public ICollection DoRDFQuery (Query query)
+		{
+			return Driver.DoRDFQuery (query, TextCache);
+		}
+#endif
+
 		public void DoQuery (Query                query,
 				     IQueryResult         query_result,
 				     IQueryableChangeData i_change_data)
@@ -310,19 +315,6 @@ namespace Beagle.Daemon {
 				if (query.IsIndexListener) {
 					ArrayList synthetic_hits = new ArrayList ();
 					foreach (Uri uri in added_uris) {
-						if (our_uri_filter != null) {
-							bool accept = false;
-
-							try {
-								accept = our_uri_filter (uri);
-							} catch (Exception e) {
-								Log.Warn (e, "Caught an exception in HitIsValid for {0}", uri);
-							}
-
-							if (! accept)
-								continue;
-						}
-
 						Hit hit = new Hit ();
 						hit.Uri = uri;
 
@@ -351,13 +343,18 @@ namespace Beagle.Daemon {
 					query_result,
 					added_uris,
 					backend_query_part_hook,
-					our_uri_filter,
 					our_hit_filter);
 		}
 
+		public int DoCountMatchQuery (Query query)
+		{
+			return Driver.DoCountMatchQuery (query, backend_query_part_hook);
+		}
+
+
 		/////////////////////////////////////////
 
-		protected SnippetReader GetSnippetFromTextCache (string [] query_terms, Uri uri, bool full_text)
+		protected SnippetReader GetSnippetFromTextCache (string [] query_terms, Uri uri, bool full_text, int ctx_length, int snp_length)
 		{
 			// Look up the hit in our text cache.  If it is there,
 			// use the cached version to generate a snippet.
@@ -367,12 +364,12 @@ namespace Beagle.Daemon {
 			if (reader == null)
 				return null;
 
-			return SnippetFu.GetSnippet (query_terms, reader, full_text);
+			return SnippetFu.GetSnippet (query_terms, reader, full_text, ctx_length, snp_length);
 		}
 
-		virtual public ISnippetReader GetSnippet (string [] query_terms, Hit hit, bool full_text)
+		virtual public ISnippetReader GetSnippet (string [] query_terms, Hit hit, bool full_text, int ctx_length, int snp_length)
 		{
-			return GetSnippetFromTextCache (query_terms, hit.Uri, full_text);
+			return GetSnippetFromTextCache (query_terms, hit.Uri, full_text, ctx_length, snp_length);
 		}
 
 		/////////////////////////////////////////
@@ -1048,5 +1045,11 @@ namespace Beagle.Daemon {
 		{
 			return new LuceneQueryingDriver (index_name, minor_version, read_only_mode);
 		}
+
+		//////////////////////////////////////////////////////////////////////////////////
+
+		virtual internal void DebugHook ()
+		{
+		}
 	}
 }
