@@ -51,7 +51,6 @@ namespace Beagle.Daemon {
 
 		static private bool Debug = false;
 
-		public delegate bool UriFilter (Uri uri);
 		public delegate double RelevancyMultiplier (Hit hit);
 
 		public LuceneQueryingDriver (string index_name, bool read_only)
@@ -69,7 +68,7 @@ namespace Beagle.Daemon {
 				// We're in read-only mode, but we can't create an index.
 				// Maybe a different exception would be better?  This one is caught
 				// in QueryDriver.LoadStaticQueryable ()
-				throw new InvalidOperationException ();
+				throw new InvalidOperationException (String.Format ("Index {0} does not exist.", index_name));
 			}
 
 			// Initialize the user text cache only if we're not in
@@ -211,15 +210,24 @@ namespace Beagle.Daemon {
 			return term_list;
 		}
 
-		private void BuildSearchers (out IndexReader primary_reader,
+		// Returns true if there are docs to search and creates the readers and searchers
+		// in that case. Otherwise, returns false.
+		private bool BuildSearchers (out IndexReader primary_reader,
 					    out LNS.IndexSearcher primary_searcher,
 					    out IndexReader secondary_reader,
 					    out LNS.IndexSearcher secondary_searcher)
 		{
+			primary_searcher = null;
 			secondary_reader = null;
 			secondary_searcher = null;
 
 			primary_reader = LuceneCommon.GetReader (PrimaryStore);
+			if (primary_reader.NumDocs() == 0) {
+				ReleaseReader (primary_reader);
+				primary_reader = null;
+				return false;
+			}
+
 			primary_searcher = new LNS.IndexSearcher (primary_reader);
 
 			if (SecondaryStore != null) {
@@ -232,6 +240,8 @@ namespace Beagle.Daemon {
 
 			if (secondary_reader != null)
 				secondary_searcher = new LNS.IndexSearcher (secondary_reader);
+
+			return true;
 		}
 
 		private void CloseSearchers (IndexReader primary_reader,
@@ -313,6 +323,445 @@ namespace Beagle.Daemon {
 			}
 		}
 
+#if ENABLE_RDF_ADAPTER
+		///////// RDF fu ///////////////////////////////////////////////
+
+		// Returns a collection of Uris
+		// HitFilter and UriFilter are ignored for now
+		// They will come into play in the final FetchDocument part
+		// FIXME: Should RDFQuery do any query mapping using backend_query_part_hook ?
+		// I think it should not. QueryPart hooks are for human beings, RDF is for softwares.
+		public ICollection DoRDFQuery (Query _query, TextCache text_cache)
+		{
+			RDFQuery query = (RDFQuery) _query;
+
+			string subject, predicate, _object;
+			PropertyType pred_type;
+
+			subject = query.SubjectString;
+			predicate = query.Predicate;
+			pred_type = query.PredicateType;
+			_object = query.Object;
+
+			if (Debug)
+				Logger.Log.Debug ("###### {0}: Starting low-level queries '{1}' : '{4}:{2}' = '{3}'", IndexName, subject, predicate, _object, pred_type);
+
+			// ******** 8 cases **********
+
+			// Return all uris
+			if (subject == String.Empty && predicate == String.Empty && _object == String.Empty) {
+				ICollection hits = GetAllHitsByUri ().Values;
+				foreach (Hit hit in hits)
+					foreach (Property text_link_property in GetTextLinks (hit.Uri, text_cache))
+						hit.AddProperty (text_link_property);
+				return hits;
+			}
+
+			// Normal query
+			if (subject == String.Empty && predicate == String.Empty && _object != String.Empty) {
+				QueryPart_Text part = new QueryPart_Text ();
+				part.Text = _object;
+				part.SearchFullText = false; // We only search properties in RDF query
+				query.AddPart (part);
+				return DoLowLevelRDFQuery (query, pred_type, predicate, _object, text_cache);
+			}
+
+			// Return uris for all documents with this property
+			if (subject == String.Empty && predicate != String.Empty && _object == String.Empty) {
+				string field_name = PropertyToFieldName (pred_type, predicate);
+
+				QueryPart_Property part = new QueryPart_Property ();
+				part.Type = PropertyType.Internal;
+				part.Key = "Properties";
+				part.Value = field_name;
+				query.AddPart (part);
+
+				return DoLowLevelRDFQuery (query, pred_type, predicate, null, text_cache);
+			}
+
+			// Property query
+			if (subject == String.Empty && predicate != String.Empty && _object != String.Empty) {
+				QueryPart_Property part = new QueryPart_Property ();
+				part.Type = pred_type;
+				part.Key = predicate;
+				part.Value = _object;
+				query.AddPart (part);
+				return DoLowLevelRDFQuery (query, pred_type, predicate, _object, text_cache);
+			}
+
+			// Return if the URI exists
+			if (subject != String.Empty && predicate == String.Empty && _object == String.Empty) {
+				QueryPart_Uri part = new QueryPart_Uri ();
+				part.Uri = UriFu.UserUritoEscapedUri (subject); // better be URI!
+				query.AddPart (part);
+				// FIXME: Which properties to return in the hit? All or none ?
+				return DoLowLevelRDFQuery (query, pred_type, predicate, null, text_cache);
+			}
+
+			// Normal query in the document with this URI
+			if (subject != String.Empty && predicate == String.Empty && _object != String.Empty) {
+				QueryPart_Uri uri_part = new QueryPart_Uri ();
+				uri_part.Uri = UriFu.UserUritoEscapedUri (subject); // better be URI!
+				query.AddPart (uri_part);
+
+				QueryPart_Text part = new QueryPart_Text ();
+				part.Text = _object;
+				part.SearchFullText = false; // We only search properties in RDF query
+				query.AddPart (part);
+
+				return DoLowLevelRDFQuery (query, pred_type, predicate, _object, text_cache);
+			}
+
+			// Return URI if the document with this URI contains this property
+			if (subject != String.Empty && predicate != String.Empty && _object == String.Empty) {
+				ArrayList returned_uris = new ArrayList (1);
+
+				ArrayList uri_list = new ArrayList (1);
+				uri_list.Add (UriFu.UserUritoEscapedUri (subject));
+
+				string field_name = PropertyToFieldName (pred_type, predicate);
+				FieldSelector fields = new MapFieldSelector (new string[] { "Uri", "Timestamp", field_name });
+				ICollection hits = GetHitsForUris (uri_list, fields);
+				if (predicate == "TextLinks") {
+					foreach (Hit hit in hits)
+						foreach (Property text_link_property in GetTextLinks (hit.Uri, text_cache))
+							hit.AddProperty (text_link_property);
+				}
+
+				return hits;
+			}
+
+			// Property query in the document with this URI
+			if (subject != String.Empty && predicate != String.Empty && _object != String.Empty) {
+				QueryPart_Uri uri_part = new QueryPart_Uri ();
+				uri_part.Uri = UriFu.UserUritoEscapedUri (subject); // better be URI!
+				query.AddPart (uri_part);
+
+				QueryPart_Property part = new QueryPart_Property ();
+				part.Type = pred_type;
+				part.Key = predicate;
+				part.Value = _object;
+				query.AddPart (part);
+
+				return DoLowLevelRDFQuery (query, pred_type, predicate, _object, text_cache);
+			}
+
+			throw new Exception ("Never reaches");
+		}
+
+		private ICollection DoLowLevelRDFQuery (Query query,
+							PropertyType pred_type,
+							string predicate,
+							string field_value,
+							TextCache text_cache)
+		{
+
+			Stopwatch total, a, b, c, d, e, f;
+
+			total = new Stopwatch ();
+			a = new Stopwatch ();
+			b = new Stopwatch ();
+			c = new Stopwatch ();
+			d = new Stopwatch ();
+			e = new Stopwatch ();
+			f = new Stopwatch ();
+
+			total.Start ();
+			a.Start ();
+
+			// Assemble all of the parts into a bunch of Lucene queries
+
+			ArrayList primary_required_part_queries;
+			ArrayList secondary_required_part_queries;
+
+			LNS.BooleanQuery primary_prohibited_part_query;
+			LNS.BooleanQuery secondary_prohibited_part_query;
+
+			AndHitFilter all_hit_filters;
+
+			ArrayList term_list;
+
+			// Assemble all of the parts into a bunch of Lucene queries
+
+			term_list = AssembleQuery (query,
+				null,
+				null,
+				out primary_required_part_queries,
+				out secondary_required_part_queries,
+				out primary_prohibited_part_query,
+				out secondary_prohibited_part_query,
+				out all_hit_filters);
+
+			a.Stop ();
+			if (Debug)
+				Log.Debug ("###### {0}: Building queries took {1}", IndexName, a);
+
+			// If we have no required parts, give up.
+			if (primary_required_part_queries == null)
+				return null;
+
+			b.Start ();
+			
+			//
+			// Now that we have all of these nice queries, let's execute them!
+			//
+
+			// Create the searchers that we will need.
+
+			IndexReader primary_reader;
+			LNS.IndexSearcher primary_searcher;
+			IndexReader secondary_reader;
+			LNS.IndexSearcher secondary_searcher;
+
+			// Create the searchers that we will need.
+			if (! BuildSearchers (out primary_reader, out primary_searcher, out secondary_reader, out secondary_searcher))
+				return null;
+
+			b.Stop ();
+			if (Debug)
+				Log.Debug ("###### {0}: Readers/searchers built in {1}", IndexName, b);
+
+			// Build whitelists and blacklists for search subsets.
+			c.Start ();
+			
+			// Possibly create our whitelists from the search subset.
+			LuceneBitArray primary_whitelist, secondary_whitelist;
+			CreateQueryWhitelists (null,
+				primary_searcher,
+				secondary_searcher,
+				primary_prohibited_part_query,
+				secondary_prohibited_part_query,
+				out primary_whitelist,
+				out secondary_whitelist);
+
+			c.Stop ();
+			if (Debug)
+				Log.Debug ("###### {0}: Whitelists and blacklists built in {1}", IndexName, c);
+
+			// Now run the low level queries against our indexes.
+			d.Start ();
+
+			BetterBitArray primary_matches = null;
+
+			if (primary_required_part_queries != null) {
+
+				if (secondary_searcher != null)
+					primary_matches = DoRequiredQueries_TwoIndex (primary_searcher,
+										      secondary_searcher,
+										      primary_required_part_queries,
+										      secondary_required_part_queries,
+										      primary_whitelist,
+										      secondary_whitelist);
+				else
+					primary_matches = DoRequiredQueries (primary_searcher,
+									     primary_required_part_queries,
+									     primary_whitelist);
+
+			} 
+
+			d.Stop ();
+			if (Debug)
+				Logger.Log.Debug ("###### {0}: Low-level queries finished in {1} and returned {2} matches", IndexName, d, primary_matches.TrueCount);
+
+			e.Start ();
+
+			int count = 0;
+			Document doc;
+			ArrayList hits = new ArrayList (primary_matches.TrueCount);
+
+			TermDocs secondary_term_docs = null;
+			if (secondary_searcher != null)
+				secondary_term_docs = secondary_searcher.Reader.TermDocs ();
+		
+			FieldSelector fields = null;
+			if (predicate != null)
+				fields = new MapFieldSelector (new string[] { "Uri", "Timestamp", PropertyToFieldName (pred_type, predicate)});
+
+			for (int match_index = primary_matches.GetNextTrueIndex (0);
+			     match_index < primary_matches.Count; 
+			     match_index = primary_matches.GetNextTrueIndex (++ match_index)) {
+
+				count++;
+
+				// If we have a HitFilter, apply it.
+				// RDF FIXME: Ignore Hit Filter for now
+
+				// If predicate was not specified but object was specified,
+				// then figure out the right predicate
+				if (predicate == null && field_value != null) {
+					Hit hit = new Hit ();
+					doc = primary_searcher.Doc (match_index);
+					hit.Uri = GetUriFromDocument (doc);
+					hit.Timestamp = StringFu.StringToDateTime (doc.Get ("Timestamp"));
+
+					bool found_matching_predicate = false;
+
+					foreach (Field field in doc.Fields ()) {
+						if (! FieldIsPredicate (field, field_value))
+							continue;
+
+						Property prop = new Property ();
+						prop.Type = pred_type;
+						prop.Key = predicate;
+						prop.Value = field_value;
+						hit.AddProperty (prop);
+
+						found_matching_predicate = true;
+					}
+
+					// Now get the matching predicate from the secondary index
+					if (secondary_searcher == null) {
+						doc = null;
+					} else {
+						Term term = new Term ("Uri", doc.Get ("Uri"));
+						secondary_term_docs.Seek (term);
+						if (secondary_term_docs.Next ())
+							doc = secondary_searcher.Doc (secondary_term_docs.Doc ());
+					}
+
+					if (doc != null) {
+						foreach (Field field in doc.Fields ()) {
+							if (! FieldIsPredicate (field, field_value))
+								continue;
+
+							Property prop = new Property ();
+							prop.Type = pred_type;
+							prop.Key = predicate;
+							prop.Value = field_value;
+							hit.AddProperty (prop);
+
+							found_matching_predicate = true;
+						}
+					}
+
+					if (! found_matching_predicate) {
+						// No matching predicate found
+						// This means some unstored field matched the query
+						// FIXME: Add a synthetic property #text
+						hit.AddProperty (Property.New ("#text", field_value));
+					}
+					
+					hits.Add (hit);
+				} else if (predicate == "TextLinks") {
+					// Special treatment: TextLinks is not stored but can be queried
+					doc = primary_searcher.Doc (match_index, fields_timestamp_uri);
+					Hit hit = CreateHit (doc, secondary_reader, secondary_term_docs, fields);
+					if (field_value != null)
+						hit.AddProperty (Property.New ("TextLinks", field_value));
+					else {
+						foreach (Property text_link_property in GetTextLinks (hit.Uri, text_cache))
+							hit.AddProperty (text_link_property);
+					}
+					hits.Add (hit);
+				} else {
+					doc = primary_searcher.Doc (match_index, fields);
+					Hit hit = CreateHit (doc, secondary_reader, secondary_term_docs, fields);
+					foreach (Property prop in hit.Properties) {
+						if (prop.Key == predicate)
+							prop.Value = field_value;
+					}
+
+					hits.Add (hit);
+				}
+			}
+
+			e.Stop ();
+
+			if (Debug)
+				Log.Debug ("###### {0}: Query results generated in {1}", IndexName, e);
+
+			//
+			// Finally, we clean up after ourselves.
+			//
+
+			f.Start ();
+			CloseSearchers (primary_reader, primary_searcher, secondary_reader, secondary_searcher);
+			f.Stop ();
+			
+			if (Debug)
+				Log.Debug ("###### {0}: Readers/searchers released in {1}", IndexName, f);
+
+			total.Stop ();
+			if (Debug) {
+				Log.Debug ("###### {0}: Query time breakdown:", IndexName);
+				Log.Debug ("###### {0}:    Build queries {1,6} ({2:0.0}%)", IndexName, a, 100 * a.ElapsedTime / total.ElapsedTime);
+				Log.Debug ("###### {0}:      Got readers {1,6} ({2:0.0}%)", IndexName, b, 100 * b.ElapsedTime / total.ElapsedTime);
+				Log.Debug ("###### {0}:       Whitelists {1,6} ({2:0.0}%)", IndexName, c, 100 * c.ElapsedTime / total.ElapsedTime);
+				Log.Debug ("###### {0}:          Queries {1,6} ({2:0.0}%)", IndexName, d, 100 * d.ElapsedTime / total.ElapsedTime);
+				Log.Debug ("###### {0}:    Gen'd Results {1,6} ({2:0.0}%)", IndexName, e, 100 * e.ElapsedTime / total.ElapsedTime);
+				Log.Debug ("###### {0}:   Reader cleanup {1,6} ({2:0.0}%)", IndexName, f, 100 * f.ElapsedTime / total.ElapsedTime);
+				Log.Debug ("###### {0}:            TOTAL {1,6}", IndexName, total);
+
+				Logger.Log.Debug ("###### {0}: Total query run in {1}", IndexName, total);
+			}
+
+			return hits;
+		}
+
+		// FIXME: This basically queries the value against the field
+		// and is really really slow!
+		private bool FieldIsPredicate (Field field, string value)
+		{
+			string field_name = field.Name ();
+			string field_value = field.StringValue ();
+			Console.WriteLine ("Reverse searching for '{0}' value in {1}='{2}'", value, field_name, field_value);
+			// Simply run the value of the property against the right analyzer
+			// and check if there is any match
+			TokenStream source = IndexingAnalyzer.TokenStream (field_name, new StringReader (field_value));
+			StringBuilder sb = new StringBuilder ();
+			try {
+				Lucene.Net.Analysis.Token token;
+				while (true) {
+					token = source.Next ();
+					if (token == null)
+						break;
+					sb.Append (token.TermText ());
+					sb.Append (" ");
+					break;
+				}
+			} finally {
+				try {
+					source.Close ();
+				} catch { }
+			}
+
+			string field_analyzed = sb.ToString ();
+			sb.Length = 0;
+
+			source = QueryAnalyzer.TokenStream (field_name, new StringReader (value));
+			try {
+				Lucene.Net.Analysis.Token token;
+				while (true) {
+					token = source.Next ();
+					if (token == null)
+						break;
+					sb.Append (token.TermText ());
+					sb.Append (" ");
+					break;
+				}
+			} finally {
+				try {
+					source.Close ();
+				} catch { }
+			}
+
+			string value_analyzed = sb.ToString ();
+			return field_analyzed.Contains (value_analyzed);
+		}
+
+		private IEnumerable GetTextLinks (Uri uri, TextCache text_cache)
+		{
+			if (text_cache == null)
+				yield break;
+
+			IList<string> links = text_cache.GetLinks (uri);
+			if (links == null)
+				yield break;
+
+			foreach (string link in links)
+				yield return Property.NewKeyword ("TextLinks", link);
+		}
+#endif
 		////////////////////////////////////////////////////////////////
 
 		public int DoCountMatchQuery (Query query, QueryPartHook query_part_hook)
@@ -351,7 +800,8 @@ namespace Beagle.Daemon {
 			IndexReader secondary_reader;
 			LNS.IndexSearcher secondary_searcher;
 
-			BuildSearchers (out primary_reader, out primary_searcher, out secondary_reader, out secondary_searcher);
+			if (! BuildSearchers (out primary_reader, out primary_searcher, out secondary_reader, out secondary_searcher))
+				return 0;
 
 			// Build whitelists and blacklists for search subsets.
 			LuceneBitArray primary_whitelist, secondary_whitelist;
@@ -401,7 +851,6 @@ namespace Beagle.Daemon {
 				     IQueryResult        result,
 				     ICollection         search_subset_uris, // should be internal uris
 				     QueryPartHook       query_part_hook,
-				     UriFilter           uri_filter,
 				     HitFilter           hit_filter)
 		{
 			if (Debug)
@@ -461,8 +910,8 @@ namespace Beagle.Daemon {
 			LNS.IndexSearcher secondary_searcher;
 
 			// Create the searchers that we will need.
-
-			BuildSearchers (out primary_reader, out primary_searcher, out secondary_reader, out secondary_searcher);
+			if (! BuildSearchers (out primary_reader, out primary_searcher, out secondary_reader, out secondary_searcher))
+				return;
 
 			b.Stop ();
 			if (Debug)
@@ -519,7 +968,6 @@ namespace Beagle.Daemon {
 						      result,
 						      term_list,
 						      query.MaxHits,
-						      uri_filter,
 						      new HitFilter (all_hit_filters.HitFilter),
 						      IndexName);
 			}
@@ -752,7 +1200,6 @@ namespace Beagle.Daemon {
 							  IQueryResult      result,
 							  ICollection       query_term_list,
 							  int               max_results,
-							  UriFilter         uri_filter,
 							  HitFilter         hit_filter,
 							  string            index_name)
 		{
@@ -783,13 +1230,16 @@ namespace Beagle.Daemon {
 			// This is used only for scoring
 			Dictionary<int, Hit> hits_by_id = new Dictionary<int, Hit> (num_hits);
 
+			int total_number_of_matches = primary_matches.TrueCount;
+
 			if (primary_matches.TrueCount > max_results)
 				final_list_of_hits = ScanRecentDocs (primary_reader,
 					secondary_reader,
 					primary_matches,
 					hits_by_id,
 					max_results,
-					uri_filter,
+					ref total_number_of_matches,
+					hit_filter,
 					index_name);
 
 			if (final_list_of_hits == null)
@@ -798,7 +1248,8 @@ namespace Beagle.Daemon {
 					primary_matches,
 					hits_by_id,
 					max_results,
-					uri_filter,
+					ref total_number_of_matches,
+					hit_filter,
 					index_name);
 
 			d.Start ();
@@ -813,8 +1264,6 @@ namespace Beagle.Daemon {
 
 			e.Start ();
 
-			int total_number_of_matches = primary_matches.TrueCount;
-
 			// 25 hits seems to be the sweet spot: anything lower
 			// and serialization overhead gets us, higher takes
 			// longer to send out.
@@ -822,19 +1271,8 @@ namespace Beagle.Daemon {
 			int sent_index = 0;
 
 			// Break up the hits into reasonably sized chunks for
-			// sending over the wire.  Also run our hit_filter, if
-			// we have one.
+			// sending over the wire.
 			for (int i = 0; i < final_list_of_hits.Count; ++i) {
-				if (hit_filter != null) {
-					Hit hit = (Hit) final_list_of_hits [i];
-					if (! hit_filter (hit)) {
-						if (Debug)
-							Log.Debug ("Filtered out {0}", hit.Uri);
-						final_list_of_hits [i] = null;
-						total_number_of_matches --;
-					}
-				}
-
 				// Flush our hits
 				if (i > 0 && i % MAX_QUEUED_HITS == 0) {
 					result.Add (final_list_of_hits.GetRange (0, MAX_QUEUED_HITS));
@@ -889,7 +1327,8 @@ namespace Beagle.Daemon {
 						    BetterBitArray	    primary_matches,
 						    Dictionary<int, Hit>    hits_by_id,
 						    int			    max_results,
-						    UriFilter		    uri_filter,
+						    ref int		    total_number_of_matches,
+						    HitFilter		    hit_filter,
 						    string		    index_name)
 		{
 			Stopwatch a = new Stopwatch ();
@@ -900,6 +1339,7 @@ namespace Beagle.Daemon {
 			ArrayList results = new ArrayList (max_results);
 			int docs_found = 0;
 			int docs_walked = 0;
+			int hit_filter_removed = 0;
 			int max_docs = (int) (primary_matches.TrueCount * 1.25);
 
 			Term term;
@@ -922,18 +1362,19 @@ namespace Beagle.Daemon {
 
 					if (primary_matches.Get (doc_id)) {
 						Document doc = primary_reader.Document (doc_id);
-						// If we have a UriFilter, apply it.
-						if (uri_filter != null) {
-							Uri uri;
-							uri = GetUriFromDocument (doc);
-							if (uri_filter (uri)) {
-								Hit hit = CreateHit (doc, secondary_reader, secondary_term_docs);
-								hits_by_id [doc_id] = hit;
-								// Add the result, last modified first
-								results.Add (hit);
-								docs_found++;
-							}
+						Hit hit = CreateHit (doc, secondary_reader, secondary_term_docs);
+
+						// If we have a HitFilter, apply it.
+						if (hit_filter != null && ! hit_filter (hit)) {
+							if (Debug)
+								Log.Debug ("Filtered out {0}", hit.Uri);
+							hit_filter_removed ++;
+							continue;
 						}
+						hits_by_id [doc_id] = hit;
+						// Add the result, last modified first
+						results.Add (hit);
+						docs_found++;
 					}
 			
 					docs_walked++;
@@ -952,6 +1393,13 @@ namespace Beagle.Daemon {
 				// Otherwise bad luck! Not all docs found
 				// Start afresh - this time traversing all results
 				results = null;
+			} else {
+				// Adjust total_number_of_matches. We need to do this to avoid scenarios like the following:
+				// max_hits = 100. Matched 100 results. But hit filter removed 30. So 70 results will be returned.
+				// We want to avoid saying "Showing top 70 of 100". Note that since we are not passing
+				// every document in the index through the hit_filter, when we say "Showing top 100 of 1234", the
+				// 1234 could actually be much less. But since max_hits was 100, that will not mislead the user.
+				total_number_of_matches -= hit_filter_removed;
 			}
 
 			a.Stop ();
@@ -970,7 +1418,8 @@ namespace Beagle.Daemon {
 							      BetterBitArray	    primary_matches,
 							      Dictionary<int, Hit>  hits_by_id,
 							      int		    max_results,
-							      UriFilter		    uri_filter,
+							      ref int		    total_number_of_matches,
+							      HitFilter		    hit_filter,
 							      string		    index_name)
 		{
 			Stopwatch b = new Stopwatch ();
@@ -1017,17 +1466,18 @@ namespace Beagle.Daemon {
 						continue;
 				}
 
-				// If we have a UriFilter, apply it.
-				if (uri_filter != null) {
-					Uri uri;
-					uri = GetUriFromDocument (doc);
-					if (! uri_filter (uri))
-						continue;
+				// Get the actual hit now
+				// doc was created with only 2 fields, so first get the complete lucene document for primary document.
+				// Also run our hit_filter now, if we have one. Since we insist of returning max_results
+				// most recent hits, any hits that would be filtered out should happen now and not later.
+				Hit hit = CreateHit (primary_reader.Document (match_index), secondary_reader, term_docs);
+				if (hit_filter != null && ! hit_filter (hit)) {
+					if (Debug)
+						Log.Debug ("Filtered out {0}", hit.Uri);
+					total_number_of_matches --;
+					continue;
 				}
 
-				// Get the actual hit now
-				// doc was created with only 2 fields, so first get the complete lucene document for primary document
-				Hit hit = CreateHit (primary_reader.Document (match_index), secondary_reader, term_docs);
 				hits_by_id [match_index] = hit;
 
 				// Add the document to the appropriate data structure.
@@ -1059,6 +1509,17 @@ namespace Beagle.Daemon {
 		private static Hit CreateHit ( Document primary_doc,
 					IndexReader secondary_reader,
 					TermDocs term_docs)
+		{
+			return CreateHit ( primary_doc,
+					secondary_reader,
+					term_docs,
+					null);
+		}
+
+		private static Hit CreateHit ( Document primary_doc,
+					IndexReader secondary_reader,
+					TermDocs term_docs,
+					FieldSelector fields)
 		{
 			Hit hit = DocumentToHit (primary_doc);
 
@@ -1072,7 +1533,10 @@ namespace Beagle.Daemon {
 
 			// Move to the first (and only) matching term doc
 			term_docs.Next ();
-			Document secondary_doc = secondary_reader.Document (term_docs.Doc ());
+			Document secondary_doc =
+				(fields == null) ?
+				secondary_reader.Document (term_docs.Doc ()) :
+				secondary_reader.Document (term_docs.Doc (), fields);
 
 			// If we are using the secondary index, now we need to
 			// merge the properties from the secondary index
