@@ -83,7 +83,24 @@ namespace Beagle.Daemon {
 			}
 		}
 
-		public ISnippetReader GetSnippet (string[] query_terms, Hit hit, bool full_text)
+#if ENABLE_RDF_ADAPTER
+		public ICollection DoRDFQuery (Query query)
+		{
+			return iqueryable.DoRDFQuery (query);
+		}
+#endif
+
+		public int DoCountMatchQuery (Query query)
+		{
+			try {
+				return iqueryable.DoCountMatchQuery (query);
+			} catch (Exception ex) {
+				Logger.Log.Warn (ex, "Caught exception calling DoCountMatchQuery on '{0}'", Name);
+				return 0;
+			}
+		}
+
+		public ISnippetReader GetSnippet (string[] query_terms, Hit hit, bool full_text, int ctx_length, int snp_length)
 		{
 			if (hit == null)
 				return null;
@@ -95,7 +112,7 @@ namespace Beagle.Daemon {
 			}
 
 			try {
-				return iqueryable.GetSnippet (query_terms, hit, full_text);
+				return iqueryable.GetSnippet (query_terms, hit, full_text, ctx_length, snp_length);
 			} catch (Exception ex) {
 				Logger.Log.Warn (ex, "Caught exception calling DoQuery on '{0}'", Name);
 			}
