@@ -394,9 +394,9 @@ namespace Beagle.Daemon.ThunderbirdQueryable {
 			indexable.MimeType = "message/rfc822";
 			indexable.Timestamp = DateTimeUtil.UnixToDateTimeUtc (Convert.ToInt64 (GetText (document, "Date")));
 			indexable.CacheContent = true;
-			indexable.Crawled = true;
+			indexable.FlushBufferCache = true;
 			indexable.SetBinaryStream (message.Stream);
-			
+
 			indexable.AddProperty (Property.NewKeyword ("fixme:client", "thunderbird"));
 			indexable.AddProperty (Property.NewKeyword ("fixme:folder", GetText (document, "Folder")));
 			indexable.AddProperty (Property.NewUnsearched ("ParentUri", GetText (document, "FolderFile")));
@@ -526,7 +526,7 @@ namespace Beagle.Daemon.ThunderbirdQueryable {
 			indexable.MimeType = "text/html";
 			indexable.Timestamp = DateTimeUtil.UnixToDateTimeUtc (Convert.ToInt64 (GetText (document, "Date")));
 			indexable.CacheContent = true;
-			indexable.Crawled = true;
+			indexable.FlushBufferCache = true;
 			
 			indexable.AddProperty (Property.NewKeyword ("fixme:client", "thunderbird"));
 			indexable.AddProperty (Property.NewKeyword ("fixme:folder", GetText (document, "Folder")));
