@@ -114,7 +114,6 @@ var beagle = {
         if(!this.checkEnv())
         {
             this.runStatus = this.RUN_BEAGLE_NOT_FOUND;
-            this.error(_("beagle_not_found"));
         }
         else
         {
@@ -127,7 +126,10 @@ var beagle = {
                 this.disable();
             }
             if(!this.FILE_UTILS.exists(this.dataPath))
-                ;// do something here ? is it safe to create the dir ?
+            {
+                var d = new Dir(this.dataPath);
+                d.create();
+            };
         }
         this.addEventListener();
         if (this.pref.get("beagle.first.run"))
@@ -212,7 +214,8 @@ var beagle = {
         if(storage_directory == "")
             storage_directory = this.ENV.get("HOME") + "/.beagle";
         if (!this.FILE_UTILS.exists (storage_directory)) {
-            alert(_("beagle_check_env_error"));
+	    var errorMsg = (_f("beagle_check_env_error", [storage_directory]));
+	    this.error(errorMsg);
             return false;
         }
         this.dataPath = storage_directory + "/ToIndex";
@@ -424,6 +427,10 @@ var beagle = {
 
 	// Tokenize the url
 	var loc = page.location;
+	if (loc.protocol == "javascript:") {
+		log("skipping javascript..");
+		return;
+	}
 	var url_tokenized = loc.host + " " + loc.port + " " + loc.pathname + " " + loc.hash + " " + loc.search;
 	url_tokenized = url_tokenized.replace(/\./g, " ").replace(/\//g, " ").replace(/&/g, " ").replace (/\+/g, " ").replace (/=/g, " ").replace(/%../g, " ");
 	meta.push("t:beagle:inuri=" + url_tokenized);
@@ -634,9 +641,12 @@ var beagle = {
                 // currently disabled  enable.
                 this.enable();
                 break;
-            default:
+	    case this.RUN_BEAGLE_NOT_FOUND:
+		alert(this.STATUS_ICON.getAttribute("tooltiptext"));
+		break;
+            default: // RUN_ERROR
                 // last run was an error, show the error
-                alert("Error running Beagle Indexer: " + this.RunStatus);
+                alert("Error running Beagle Indexer.");
                 break;
             }
         }
